"""
Dialog for importing cloud projects
"""

from ..ui.import_cloud_project_dialog_ui import Ui_ImportCloudProjectDialog
from ..qt import QtGui
from ..cloud.utils import get_cloud_projects, DownloadProjectThread, DeleteProjectThread
from ..utils.progress_dialog import ProgressDialog


class ImportCloudProjectDialog(QtGui.QDialog, Ui_ImportCloudProjectDialog):

    """
    Import cloud project dialog implementation.
    """

    def __init__(self, parent, project_dest_path, images_dest_path, cloud_settings):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.project_dest_path = project_dest_path
        self.images_dest_path = images_dest_path
        self.cloud_settings = cloud_settings

        self.uiImportProjectAction.clicked.connect(self._importProject)
        self.uiDeleteProjectAction.clicked.connect(self._deleteProject)
        self._listCloudProjects()

    def _listCloudProjects(self):
        self.listWidget.clear()
        self.projects = get_cloud_projects(self.cloud_settings)
        self.listWidget.addItems(list(self.projects.keys()))

    def _importProject(self):
        project_file_name = self.projects[self.listWidget.currentItem().text()]

        download_thread = DownloadProjectThread(
            self,
            project_file_name,
            self.project_dest_path,
            self.images_dest_path,
            self.cloud_settings
        )
        progress_dialog = ProgressDialog(download_thread, "Importing project", "Downloading project files...", "Cancel",
                                         parent=self.parent())

        progress_dialog.show()
        progress_dialog.exec_()

        self.close()

    def _deleteProject(self):
        project_file_name = self.projects[self.listWidget.currentItem().text()]

        button_clicked = QtGui.QMessageBox.question(
            self,
            "Delete project",
            "Are you sure you want to delete project " + self.listWidget.currentItem().text(),
            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
            QtGui.QMessageBox.Yes
        )

        if button_clicked == QtGui.QMessageBox.Yes:
            delete_project_thread = DeleteProjectThread(self, project_file_name, self.cloud_settings)
            progress_dialog = ProgressDialog(delete_project_thread, "Deleting project", "Deleting project files...",
                                             "Cancel", parent=self)
            progress_dialog.show()
            progress_dialog.exec_()

            self._listCloudProjects()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/configuration_dialog.ui'
#
# Created: Thu Oct  2 15:12:03 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_configurationDialog(object):

    def setupUi(self, configurationDialog):
        configurationDialog.setObjectName(_fromUtf8("configurationDialog"))
        configurationDialog.resize(585, 454)
        configurationDialog.setModal(True)
        self.gridlayout = QtGui.QGridLayout(configurationDialog)
        self.gridlayout.setObjectName(_fromUtf8("gridlayout"))
        self.splitter = QtGui.QSplitter(configurationDialog)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName(_fromUtf8("splitter"))
        self.verticalLayout = QtGui.QWidget(self.splitter)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.vboxlayout = QtGui.QVBoxLayout(self.verticalLayout)
        self.vboxlayout.setSpacing(4)
        self.vboxlayout.setMargin(0)
        self.vboxlayout.setObjectName(_fromUtf8("vboxlayout"))
        self.uiTitleLabel = QtGui.QLabel(self.verticalLayout)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.uiTitleLabel.setFont(font)
        self.uiTitleLabel.setFrameShape(QtGui.QFrame.Box)
        self.uiTitleLabel.setFrameShadow(QtGui.QFrame.Sunken)
        self.uiTitleLabel.setTextFormat(QtCore.Qt.PlainText)
        self.uiTitleLabel.setObjectName(_fromUtf8("uiTitleLabel"))
        self.vboxlayout.addWidget(self.uiTitleLabel)
        self.uiConfigStackedWidget = QtGui.QStackedWidget(self.verticalLayout)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConfigStackedWidget.sizePolicy().hasHeightForWidth())
        self.uiConfigStackedWidget.setSizePolicy(sizePolicy)
        self.uiConfigStackedWidget.setFrameShape(QtGui.QFrame.Box)
        self.uiConfigStackedWidget.setFrameShadow(QtGui.QFrame.Sunken)
        self.uiConfigStackedWidget.setObjectName(_fromUtf8("uiConfigStackedWidget"))
        self.uiEmptyPageWidget = QtGui.QWidget()
        self.uiEmptyPageWidget.setObjectName(_fromUtf8("uiEmptyPageWidget"))
        self.vboxlayout1 = QtGui.QVBoxLayout(self.uiEmptyPageWidget)
        self.vboxlayout1.setSpacing(0)
        self.vboxlayout1.setContentsMargins(0, 4, 0, 0)
        self.vboxlayout1.setObjectName(_fromUtf8("vboxlayout1"))
        self.uiConfigStackedWidget.addWidget(self.uiEmptyPageWidget)
        self.vboxlayout.addWidget(self.uiConfigStackedWidget)
        self.gridlayout.addWidget(self.splitter, 0, 0, 1, 1)
        self.uiButtonBox = QtGui.QDialogButtonBox(configurationDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName(_fromUtf8("uiButtonBox"))
        self.gridlayout.addWidget(self.uiButtonBox, 1, 0, 1, 1)

        self.retranslateUi(configurationDialog)
        self.uiConfigStackedWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(configurationDialog)

    def retranslateUi(self, configurationDialog):
        configurationDialog.setWindowTitle(_translate("configurationDialog", "Configuration", None))
        self.uiTitleLabel.setText(_translate("configurationDialog", "Configuration", None))

from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/qemu/ui/qemu_preferences_page.ui'
#
# Created: Mon Mar  9 18:00:29 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QemuPreferencesPageWidget(object):
    def setupUi(self, QemuPreferencesPageWidget):
        QemuPreferencesPageWidget.setObjectName(_fromUtf8("QemuPreferencesPageWidget"))
        QemuPreferencesPageWidget.resize(374, 495)
        self.verticalLayout = QtGui.QVBoxLayout(QemuPreferencesPageWidget)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.uiTabWidget = QtGui.QTabWidget(QemuPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName(_fromUtf8("uiTabWidget"))
        self.uiServerSettingsTabWidget = QtGui.QWidget()
        self.uiServerSettingsTabWidget.setObjectName(_fromUtf8("uiServerSettingsTabWidget"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.uiServerSettingsTabWidget)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.uiUseLocalServercheckBox = QtGui.QCheckBox(self.uiServerSettingsTabWidget)
        self.uiUseLocalServercheckBox.setChecked(True)
        self.uiUseLocalServercheckBox.setObjectName(_fromUtf8("uiUseLocalServercheckBox"))
        self.verticalLayout_2.addWidget(self.uiUseLocalServercheckBox)
        spacerItem = QtGui.QSpacerItem(20, 455, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiServerSettingsTabWidget, _fromUtf8(""))
        self.verticalLayout.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        spacerItem1 = QtGui.QSpacerItem(254, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtGui.QPushButton(QemuPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName(_fromUtf8("uiRestoreDefaultsPushButton"))
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(QemuPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(QemuPreferencesPageWidget)

    def retranslateUi(self, QemuPreferencesPageWidget):
        QemuPreferencesPageWidget.setWindowTitle(_translate("QemuPreferencesPageWidget", "QEMU", None))
        self.uiUseLocalServercheckBox.setText(_translate("QemuPreferencesPageWidget", "Use the local server", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiServerSettingsTabWidget), _translate("QemuPreferencesPageWidget", "General settings", None))
        self.uiRestoreDefaultsPushButton.setText(_translate("QemuPreferencesPageWidget", "Restore defaults", None))




import os
import sys
import time
from optparse import OptionParser
import getpass

import tunnel

HELP = """\
Set up a forward tunnel across an SSH server, using paramiko. A local port
(given with -p) is forwarded across an SSH session to an address:port from
the SSH server. This is similar to the openssh -L option.
"""

SSH_PORT = 22


def get_host_port(spec, default_port):
    "parse 'hostname:22' into a host and port, with the port optional"
    args = (spec.split(':', 1) + [default_port])[:2]
    args[1] = int(args[1])
    return args[0], args[1]


def parse_options():
    global g_verbose

    parser = OptionParser(usage='usage: %prog [options] <ssh-server>[:<server-port>]',
                          version='%prog 1.0', description=HELP)
    parser.add_option('-u', '--user', action='store', type='string', dest='user',
                      default=getpass.getuser(),
                      help='username for SSH authentication (default: %s)' % getpass.getuser())
    parser.add_option('-K', '--key', action='store', type='string', dest='keyfile',
                      default=None,
                      help='private key file to use for SSH authentication')
    parser.add_option('-r', '--remote', action='store', type='string', dest='remote', default=None, metavar='host:port',
                      help='remote host and port to forward to')
    options, args = parser.parse_args()

    if len(args) != 1:
        parser.error('Incorrect number of arguments.')
    if options.remote is None:
        parser.error('Remote address required (-r).')

    server_host, server_port = get_host_port(args[0], SSH_PORT)
    remote_host, remote_port = get_host_port(options.remote, SSH_PORT)
    return options, (server_host, server_port), (remote_host, remote_port)


def main():
    options, server, remote = parse_options()

    if options.keyfile:
        my_key = open(options.keyfile, 'r')

    t = tunnel.Tunnel(server[0], server[1], username=options.user, client_key=my_key)

    print("Adding defaults ...")
    t1 = t.add_endpoint('127.0.0.1', 25)
    t2 = t.add_endpoint('127.0.0.1', 80)

    print("Adding command line destination")
    t3 = t.add_endpoint(remote[0], remote[1])

    print("Removing defaults ...")
    t.remove_endpoint(t1)
    t.remove_endpoint(t2)

    t.disconnect()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import json
from .version import __version__

import logging
log = logging.getLogger(__name__)


class LocalConfig:

    """
    Handles the local GUI settings.
    """

    def __init__(self):

        self._settings = {}

        if sys.platform.startswith("win"):
            filename = "gns3_gui.ini"
        else:
            filename = "gns3_gui.conf"

        if sys.platform.startswith("darwin"):
            appname = "gns3.net"
        else:
            appname = "GNS3"

        if sys.platform.startswith("win"):

            # On windows, the system wide configuration file location is %COMMON_APPDATA%/GNS3/gns3_gui.conf
            common_appdata = os.path.expandvars("%COMMON_APPDATA%")
            system_wide_config_file = os.path.join(common_appdata, appname, filename)

            # On windows, the user specific configuration file location is %APPDATA%/GNS3/gns3_gui.conf
            appdata = os.path.expandvars("%APPDATA%")
            self._config_file = os.path.join(appdata, appname, filename)

        else:

            # On UNIX-like platforms, the system wide configuration file location is /etc/xdg/GNS3/gns3_gui.conf
            system_wide_config_file = os.path.join("/etc/xdg", appname, filename)

            # On UNIX-like platforms, the user specific configuration file location is /etc/xdg/GNS3/gns3_gui.conf
            home = os.path.expanduser("~")
            self._config_file = os.path.join(home, ".config", appname, filename)

        # First load system wide settings
        if os.path.exists(system_wide_config_file):
            self._settings = self._readConfig(system_wide_config_file)
            if not self._settings:
                log.warning("No system wide settings loaded from {}".format(system_wide_config_file))

        config_file_in_cwd = os.path.join(os.getcwd(), filename)
        if os.path.exists(config_file_in_cwd):
            # use any config file present in the current working directory
            self._config_file = config_file_in_cwd
        elif not os.path.exists(self._config_file):
            try:
                # create the config file if it doesn't exist
                os.makedirs(os.path.dirname(self._config_file), exist_ok=True)
                with open(self._config_file, "w") as f:
                    json.dump({"version": __version__, "type": "settings"}, f)
            except OSError as e:
                log.error("Could not create the config file {}: {}".format(self._config_file, e))

        user_settings = self._readConfig(self._config_file)
        # overwrite system wide settings with user specific ones
        self._settings.update(user_settings)
        self._writeConfig()

    def _readConfig(self, config_path):
        """
        Read the configuration file.
        """

        try:
            with open(config_path, "r") as f:
                return json.load(f)
        except (ValueError, OSError) as e:
            log.error("Could not read the config file {}: {}".format(self._config_file, e))

        return dict()

    def _writeConfig(self):
        """
        Write the configuration file.
        """

        try:
            with open(self._config_file, "w") as f:
                json.dump(self._settings, f, sort_keys=True, indent=4)
        except (ValueError, OSError) as e:
            log.error("Could not write the config file {}: {}".format(self._config_file, e))

    def configFilePath(self):
        """
        Returns the config file path.

        :returns: path to the config file.
        """

        return self._config_file

    def setConfigFilePath(self, config_file):
        """
        Set a new config file

        :returns: path to the config file.
        """

        self._settings = self._readConfig(self._config_file)
        self._config_file = config_file

    def settings(self):
        """
        Get the settings.

        :returns: settings (dict)
        """

        return self._readConfig(self._config_file)

    def setSettings(self, settings):
        """
        Save the settings.

        :param settings: settings to save (dict)
        """

        self._settings.update(settings)
        self._writeConfig()

    def loadSectionSettings(self, section, default_settings):
        """
        Get all the settings from a given section.

        :param default_settings: setting names and default values (dict)

        :returns: settings (dict)
        """

        settings = self.settings().get(section, dict())

        # use default values for missing settings
        for name, value in default_settings.items():
            if name not in settings:
                settings[name] = value

        if section not in self._settings:
            self._settings[section] = {}
        self._settings[section].update(settings)
        return settings

    def saveSectionSettings(self, section, settings):
        """
        Save all the settings in a given section.

        :param section: section name
        :param settings: settings to save (dict)
        """

        if section not in self._settings:
            self._settings[section] = {}
        self._settings[section].update(settings)
        self._writeConfig()

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of LocalConfig.

        :returns: instance of LocalConfig
        """

        if not hasattr(LocalConfig, "_instance"):
            LocalConfig._instance = LocalConfig()
        return LocalConfig._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VirtualBox module implementation.
"""

import os
import sys
import shutil

from gns3.qt import QtCore, QtGui
from gns3.local_server_config import LocalServerConfig
from gns3.local_config import LocalConfig

from ..module import Module
from ..module_error import ModuleError
from .virtualbox_vm import VirtualBoxVM
from .settings import VBOX_SETTINGS, VBOX_SETTING_TYPES
from .settings import VBOX_VM_SETTINGS, VBOX_VM_SETTING_TYPES

import logging
log = logging.getLogger(__name__)


class VirtualBox(Module):

    """
    VirtualBox module.
    """

    def __init__(self):
        Module.__init__(self)

        self._settings = {}
        self._virtualbox_vms = {}
        self._nodes = []

        # load the settings
        self._loadSettings()
        self._loadVirtualBoxVMs()

    @staticmethod
    def _findVBoxManage(self):
        """
        Finds the VBoxManage path.

        :return: path to VBoxManage
        """

        if sys.platform.startswith("win"):
            if "VBOX_INSTALL_PATH" in os.environ:
                vboxmanage_path = os.path.join(os.environ["VBOX_INSTALL_PATH"], "VBoxManage.exe")
            elif "VBOX_MSI_INSTALL_PATH" in os.environ:
                vboxmanage_path = os.path.join(os.environ["VBOX_MSI_INSTALL_PATH"], "VBoxManage.exe")
            else:
                vboxmanage_path = "VBoxManage.exe"
        elif sys.platform.startswith("darwin"):
            vboxmanage_path = "/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"
        else:
            vboxmanage_path = shutil.which("vboxmanage")

        if vboxmanage_path is None:
            return ""
        return vboxmanage_path

    def _loadSettings(self):
        """
        Loads the settings from the server settings file.
        """

        local_config = LocalConfig.instance()

        # restore the VirtualBox settings from QSettings (for backward compatibility)
        legacy_settings = {}
        settings = QtCore.QSettings()
        settings.beginGroup(self.__class__.__name__)
        for name in VBOX_SETTINGS.keys():
            if settings.contains(name):
                legacy_settings[name] = settings.value(name, type=VBOX_SETTING_TYPES[name])
        settings.remove("")
        settings.endGroup()

        if legacy_settings:
            local_config.saveSectionSettings(self.__class__.__name__, legacy_settings)
        self._settings = local_config.loadSectionSettings(self.__class__.__name__, VBOX_SETTINGS)

        if not os.path.exists(self._settings["vboxmanage_path"]):
            self._settings["vboxmanage_path"] = self._findVBoxManage(self)

        # keep the config file sync
        self._saveSettings()

    def _saveSettings(self):
        """
        Saves the settings to the server settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # save some settings to the local server config file
        server_settings = {
            "vbox_user": self._settings["vbox_user"],
        }

        if self._settings["vboxmanage_path"]:
            server_settings["vboxmanage_path"] = os.path.normpath(self._settings["vboxmanage_path"])

        config = LocalServerConfig.instance()
        config.saveSettings(self.__class__.__name__, server_settings)

    def _loadVirtualBoxVMs(self):
        """
        Load the VirtualBox VMs from the client settings file.
        """

        local_config = LocalConfig.instance()

        # restore the VirtualBox settings from QSettings (for backward compatibility)
        virtualbox_vms = []
        # load the settings
        settings = QtCore.QSettings()
        settings.beginGroup("VirtualBoxVMs")
        # load the VMs
        size = settings.beginReadArray("VM")
        for index in range(0, size):
            settings.setArrayIndex(index)
            vm = {}
            for setting_name, default_value in VBOX_VM_SETTINGS.items():
                vm[setting_name] = settings.value(setting_name, default_value, VBOX_VM_SETTING_TYPES[setting_name])
            virtualbox_vms.append(vm)
        settings.endArray()
        settings.remove("")
        settings.endGroup()

        if virtualbox_vms:
            local_config.saveSectionSettings(self.__class__.__name__, {"vms": virtualbox_vms})

        settings = local_config.settings()
        if "vms" in settings.get(self.__class__.__name__, {}):
            for vm in settings[self.__class__.__name__]["vms"]:
                vmname = vm.get("vmname")
                server = vm.get("server")
                key = "{server}:{vmname}".format(server=server, vmname=vmname)
                if key in self._virtualbox_vms or not vmname or not server:
                    continue
                vm_settings = VBOX_VM_SETTINGS.copy()
                vm_settings.update(vm)
                self._virtualbox_vms[key] = vm_settings

        # keep things sync
        self._saveVirtualBoxVMs()

    def _saveVirtualBoxVMs(self):
        """
        Saves the VirtualBox VMs to the client settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, {"vms": list(self._virtualbox_vms.values())})

    def virtualBoxVMs(self):
        """
        Returns VirtualBox VMs settings.

        :returns: VirtualBox VMs settings (dictionary)
        """

        return self._virtualbox_vms

    def setVirtualBoxVMs(self, new_virtualbox_vms):
        """
        Sets VirtualBox VM settings.

        :param new_iou_images: IOS images settings (dictionary)
        """

        self._virtualbox_vms = new_virtualbox_vms.copy()
        self._saveVirtualBoxVMs()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def createNode(self, node_class, server, project):
        """
        Creates a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        log.info("creating node {}".format(node_class))

        # create an instance of the node class
        return node_class(self, server, project)

    def setupNode(self, node, node_name):
        """
        Setups a node.

        :param node: Node instance
        :param node_name: Node name
        """

        log.info("configuring node {} with id {}".format(node, node.id()))

        vm = None
        if node_name:
            for vm_key, info in self._virtualbox_vms.items():
                if node_name == info["vmname"]:
                    vm = vm_key

        if not vm:
            selected_vms = []
            for vm, info in self._virtualbox_vms.items():
                if info["server"] == node.server().host or (node.server().isLocal() and info["server"] == "local"):
                    selected_vms.append(vm)

            if not selected_vms:
                raise ModuleError("No VirtualBox VM on server {}".format(node.server().host))
            elif len(selected_vms) > 1:

                from gns3.main_window import MainWindow
                mainwindow = MainWindow.instance()

                (selection, ok) = QtGui.QInputDialog.getItem(mainwindow, "VirtualBox VM", "Please choose a VM", selected_vms, 0, False)
                if ok:
                    vm = selection
                else:
                    raise ModuleError("Please select a VirtualBox VM")

            else:
                vm = selected_vms[0]

        linked_base = self._virtualbox_vms[vm]["linked_base"]
        if not linked_base:
            for other_node in self._nodes:
                if other_node.settings()["vmname"] == self._virtualbox_vms[vm]["vmname"] and \
                        (self._virtualbox_vms[vm]["server"] == "local" and other_node.server().isLocal() or self._virtualbox_vms[vm]["server"] == other_node.server().host):
                    raise ModuleError("Sorry a VirtualBox VM can only be used once in your topology (this will change in future versions)")

        settings = {}
        for setting_name, value in self._virtualbox_vms[vm].items():
            if setting_name in node.settings():
                settings[setting_name] = value

        vmname = self._virtualbox_vms[vm]["vmname"]
        node.setup(vmname, linked_clone=linked_base, additional_settings=settings)

    def reset(self):
        """
        Resets the module.
        """

        log.info("VirtualBox module reset")
        self._nodes.clear()

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [VirtualBoxVM]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for vbox_vm in self._virtualbox_vms.values():
            nodes.append(
                {"class": VirtualBoxVM.__name__,
                 "name": vbox_vm["vmname"],
                 "server": vbox_vm["server"],
                 "categories": VirtualBoxVM.categories(),
                 "default_symbol": vbox_vm["default_symbol"],
                 "hover_symbol": vbox_vm["hover_symbol"],
                 "categories": [vbox_vm["category"]]}
            )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.virtualbox_preferences_page import VirtualBoxPreferencesPage
        from .pages.virtualbox_vm_preferences_page import VirtualBoxVMPreferencesPage
        return [VirtualBoxPreferencesPage, VirtualBoxVMPreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of VirtualBox module.

        :returns: instance of VirtualBox
        """

        if not hasattr(VirtualBox, "_instance"):
            VirtualBox._instance = VirtualBox()
        return VirtualBox._instance



import importlib


def import_from_string(string_val):
    """
    Attempt to import a name from its string representation.
    """
    try:
        parts = string_val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except ImportError:
        msg = "Could not import '%s'." % string_val
        raise ImportError(msg)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from .qt import QtGui, QtCore, QtWebKit
from .ui.news_dock_widget_ui import Ui_NewsDockWidget
from .utils.get_resource import get_resource

import logging
log = logging.getLogger(__name__)


class NewsDockWidget(QtGui.QDockWidget, Ui_NewsDockWidget):

    """
    :param parent: parent widget
    """

    def __init__(self, parent):

        QtGui.QDockWidget.__init__(self, parent)
        self.setupUi(self)

        self._visible = True
        self.visibilityChanged.connect(self._visibilityChangedSlot)
        self.uiWebView.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateAllLinks)
        self.uiWebView.page().mainFrame().setScrollBarPolicy(QtCore.Qt.Vertical, QtCore.Qt.ScrollBarAlwaysOff)
        self.uiWebView.linkClicked.connect(self._urlClickedSlot)
        self.uiWebView.loadFinished.connect(self._loadFinishedSlot)
        self._refresh_timer = QtCore.QTimer(self)
        self._refresh_timer.timeout.connect(self._refreshSlot)
        self._refresh_timer.start(300000)
        self._timer = QtCore.QTimer(self)
        self._timer.timeout.connect(self._loadFinishedSlot)
        self._timer.setSingleShot(True)
        self._timer.start(5000)
        if parent.settings()["default_local_news"]:
            self._loadFinishedSlot()
        else:
            self.uiWebView.load(QtCore.QUrl("http://as.gns3.com/software/docked_200x200.html"))

    def _visibilityChangedSlot(self, visible):
        """
        Slot for visibility changed signal.

        :param visible: either the dock is visible or not
        """

        self._visible = visible

    def isVisible(self):

        return self._visible

    def closeEvent(self, event):
        """
        You really cannot close that dock (using ATL+F4...)

        :param event: closeEvent instance.
        """

        event.ignore()

    def _refreshSlot(self):
        """
        Refeshes the page.
        """

        self.uiWebView.reload()

    def _urlClickedSlot(self, url):
        """
        Opens a clicked URL using user's default browser.

        :param url: URL to open
        """

        if QtGui.QDesktopServices.openUrl(url) is False:
            QtGui.QMessageBox.critical(self, "Getting started", "Failed to open the URL: {}".format(url))

    def _loadFinishedSlot(self, result=False):
        """
        Slot called when the web page has been loaded.

        :param result: boolean
        """

        self.uiWebView.loadFinished.disconnect(self._loadFinishedSlot)
        self._timer.stop()
        self._timer.timeout.disconnect()
        if result is False:
            self._refresh_timer.stop()
            # load a local resource if the page is not available
            gns3_jungle = get_resource(os.path.join("static", "gns3_jungle.html"))
            if gns3_jungle and not (sys.platform.startswith("win") and not sys.maxsize > 2 ** 32):
                # do not show the page on Windows 32-bit (crash when no Internet connection)
                self.uiWebView.load(QtCore.QUrl.fromLocalFile(gns3_jungle))
            else:
                self.hide()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Default general settings.
"""

import os
import sys
import platform

# Projects directory location
DEFAULT_PROJECTS_PATH = os.path.normpath(os.path.expanduser("~/GNS3/projects"))

# Images directory location
DEFAULT_IMAGES_PATH = os.path.normpath(os.path.expanduser("~/GNS3/images"))

DEFAULT_LOCAL_SERVER_HOST = "127.0.0.1"
DEFAULT_LOCAL_SERVER_PORT = 8000

# Pre-configured Telnet console commands on various OSes
if sys.platform.startswith("win"):
    if "PROGRAMFILES(X86)" in os.environ:
        # windows 64-bit
        program_files = os.environ["PROGRAMFILES"]
        program_files_x86 = os.environ["PROGRAMFILES(X86)"]
    else:
        # windows 32-bit
        program_files_x86 = program_files = os.environ["PROGRAMFILES"]

    PRECONFIGURED_TELNET_CONSOLE_COMMANDS = {'Putty (included with GNS3)': 'putty.exe -telnet %h %p -wt "%d" -gns3 5 -skin 4',
                                             'SuperPutty (included with GNS3)': r'SuperPutty.exe -telnet "%h -P %p -wt \"%d\""',
                                             'SecureCRT': r'"{}\VanDyke Software\SecureCRT\SecureCRT.exe" /SCRIPT securecrt.vbs /ARG "%d" /T /TELNET %h %p'.format(program_files),
                                             'TeraTerm Pro': r'"{}\teraterm\ttermpro.exe" /W="%d" /M="ttstart.macro" /T=1 %h %p'.format(program_files_x86),
                                             'Telnet': 'telnet %h %p',
                                             'Xshell 4': r'"{}\NetSarang\Xshell 4\xshell.exe" -url telnet://%h:%p'.format(program_files_x86),
                                             'ZOC 6': r'"{}\ZOC6\zoc.exe" "/TELNET:%h:%p" /TABBED "/TITLE:%d"'.format(program_files_x86)}

    # default on Windows
    DEFAULT_TELNET_CONSOLE_COMMAND = PRECONFIGURED_TELNET_CONSOLE_COMMANDS["Putty (included with GNS3)"]

elif sys.platform.startswith("darwin"):
    # Mac OS X
    PRECONFIGURED_TELNET_CONSOLE_COMMANDS = {
        'Terminal': "osascript -e 'tell application \"Terminal\"'"
                    " -e 'activate'"
                    " -e 'set _tab to do script \"telnet %h %p ; exit\"'"
                    " -e 'delay 1'"
                    " -e 'repeat while _tab exists'"
                    " -e 'delay 1'"
                    " -e 'end repeat'"
                    " -e 'end tell'",
        'iTerm': "osascript -e 'tell application \"iTerm\"'"
                 " -e 'activate'"
                 " -e 'if (count of terminals) = 0 then'"
                 " -e '  set t to (make new terminal)'"
                 " -e 'else'"
                 " -e '  set t to current terminal'"
                 " -e 'end if'"
                 " -e 'tell t'"
                 " -e '  set s to (make new session at the end of sessions)'"
                 " -e '  tell s'"
                 " -e '    exec command (\"telnet %h %p\")'"
                 " -e '    delay 1'"
                 " -e '    repeat while s exists'"
                 " -e '      delay 1'"
                 " -e '    end repeat'"
                 " -e '  end tell'"
                 " -e 'end tell'"
                 " -e 'end tell'",
        'SecureCRT': '/Applications/SecureCRT.app/Contents/MacOS/SecureCRT /ARG "%d" /T /TELNET %h %p',
        'ZOC 6': '/Applications/zoc6.app/Contents/MacOS/zoc6 "/TELNET:%h:%p" /TABBED "/TITLE:%d"'
    }

    # default Mac OS X Telnet console command
    DEFAULT_TELNET_CONSOLE_COMMAND = PRECONFIGURED_TELNET_CONSOLE_COMMANDS["Terminal"]

else:
    PRECONFIGURED_TELNET_CONSOLE_COMMANDS = {'Xterm': 'xterm -T "%d" -e "telnet %h %p"',
                                             'Putty': 'putty -telnet %h %p -title "%d" -sl 2500 -fg SALMON1 -bg BLACK',
                                             'Gnome Terminal': 'gnome-terminal -t "%d" -e "telnet %h %p"',
                                             'Xfce4 Terminal': 'xfce4-terminal -T "%d" -e "telnet %h %p"',
                                             'ROXTerm': 'roxterm -n "%d" --tab -e "telnet %h %p"',
                                             'KDE Konsole': 'konsole --new-tab -p tabtitle="%d" -e "telnet %h %p"',
                                             'SecureCRT': 'SecureCRT /T /N "%d"  /TELNET %h %p',
                                             'Mate Terminal': 'mate-terminal --tab -e "telnet %h %p"  -t "%d"'}

    # default Telnet console command on other systems
    DEFAULT_TELNET_CONSOLE_COMMAND = PRECONFIGURED_TELNET_CONSOLE_COMMANDS["Xterm"]

    if sys.platform.startswith("linux"):
        distro = platform.linux_distribution()[0]
        if distro == "Debian" or distro == "Ubuntu" or distro == "LinuxMint":
            DEFAULT_TELNET_CONSOLE_COMMAND = PRECONFIGURED_TELNET_CONSOLE_COMMANDS["Gnome Terminal"]

# Pre-configured serial console commands on various OSes
if sys.platform.startswith("win"):
    # Windows
    PRECONFIGURED_SERIAL_CONSOLE_COMMANDS = {'Putty (included with GNS3)': 'putty.exe -serial %s -wt "%d [Local Console]" -gns3 5',
                                             'SuperPutty': r'SuperPutty.exe -serial "%s -wt \"%d\""'}

    # default Windows serial console command
    if os.path.exists(os.getcwd() + os.sep + "SuperPutty.exe"):
        DEFAULT_SERIAL_CONSOLE_COMMAND = PRECONFIGURED_SERIAL_CONSOLE_COMMANDS["SuperPutty"]
    else:
        DEFAULT_SERIAL_CONSOLE_COMMAND = PRECONFIGURED_SERIAL_CONSOLE_COMMANDS["Putty (included with GNS3)"]

elif sys.platform.startswith("darwin"):
    # Mac OS X
    PRECONFIGURED_SERIAL_CONSOLE_COMMANDS = {'Terminal + socat': "/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"socat UNIX-CONNECT:\\\"%s\\\" stdio,raw,echo=0 ; exit\"'"}

    # default Mac OS X serial console command
    DEFAULT_SERIAL_CONSOLE_COMMAND = PRECONFIGURED_SERIAL_CONSOLE_COMMANDS["Terminal + socat"]

else:
    PRECONFIGURED_SERIAL_CONSOLE_COMMANDS = {'Xterm + socat': 'xterm -T "%d" -e \'socat UNIX-CONNECT:"%s" stdio,raw,echo=0\'',
                                             'Gnome Terminal + socat': 'gnome-terminal -t "%d" -e \'socat UNIX-CONNECT:"%s" stdio,raw,echo=0\'',
                                             'Konsole + socat': 'konsole --new-tab -p tabtitle="%d" -e \'socat UNIX-CONNECT:"%s" stdio,raw,echo=0\''}

    # default serial console command on other systems
    DEFAULT_SERIAL_CONSOLE_COMMAND = PRECONFIGURED_SERIAL_CONSOLE_COMMANDS["Xterm + socat"]

    if sys.platform.startswith("linux"):
        distro = platform.linux_distribution()[0]
        if distro == "Debian" or distro == "Ubuntu" or distro == "LinuxMint":
            DEFAULT_SERIAL_CONSOLE_COMMAND = PRECONFIGURED_SERIAL_CONSOLE_COMMANDS["Gnome Terminal + socat"]

# Pre-configured packet capture reader commands on various OSes
WIRESHARK_NORMAL_CAPTURE = "Wireshark Traditional Capture"
WIRESHARK_LIVE_TRAFFIC_CAPTURE = "Wireshark Live Traffic Capture"

if sys.platform.startswith("win"):
    PRECONFIGURED_PACKET_CAPTURE_READER_COMMANDS = {WIRESHARK_NORMAL_CAPTURE: "{}\Wireshark\wireshark.exe %c".format(os.environ["PROGRAMFILES"]),
                                                    WIRESHARK_LIVE_TRAFFIC_CAPTURE: 'tail.exe -f -c +0b %c | "{}\Wireshark\wireshark.exe" -k -i -'.format(os.environ["PROGRAMFILES"])}

elif sys.platform.startswith("darwin"):
    # Mac OS X
    PRECONFIGURED_PACKET_CAPTURE_READER_COMMANDS = {WIRESHARK_NORMAL_CAPTURE: "/usr/bin/open -a /Applications/Wireshark.app %c",
                                                    WIRESHARK_LIVE_TRAFFIC_CAPTURE: "tail -f -c +0 %c | /Applications/Wireshark.app/Contents/Resources/bin/wireshark -k -i -"}

elif sys.platform.startswith("freebsd"):
    # FreeBSD
    PRECONFIGURED_PACKET_CAPTURE_READER_COMMANDS = {WIRESHARK_NORMAL_CAPTURE: "wireshark %c",
                                                    WIRESHARK_LIVE_TRAFFIC_CAPTURE: "gtail -f -c +0b %c | wireshark -k -i -"}
else:
    PRECONFIGURED_PACKET_CAPTURE_READER_COMMANDS = {WIRESHARK_NORMAL_CAPTURE: "wireshark %c",
                                                    WIRESHARK_LIVE_TRAFFIC_CAPTURE: "tail -f -c +0b %c | wireshark -k -i -"}

DEFAULT_PACKET_CAPTURE_READER_COMMAND = PRECONFIGURED_PACKET_CAPTURE_READER_COMMANDS[WIRESHARK_LIVE_TRAFFIC_CAPTURE]

DEFAULT_PACKET_CAPTURE_ANALYZER_COMMAND = ""
if sys.platform.startswith("win") and "PROGRAMFILES(X86)" in os.environ:
    # Windows 64-bit
    DEFAULT_PACKET_CAPTURE_ANALYZER_COMMAND = r'"{}\SolarWinds\ResponseTimeViewer\ResponseTimeViewer.exe" %c'.format(os.environ["PROGRAMFILES(X86)"])

STYLES = ["Charcoal", "Classic", "Legacy"]

if sys.platform.startswith("win"):
    DEFAULT_STYLE = "Classic"
else:
    DEFAULT_STYLE = "Charcoal"

GENERAL_SETTINGS = {
    "style": DEFAULT_STYLE,
    "auto_launch_project_dialog": True,
    "auto_screenshot": True,
    "check_for_update": True,
    "last_check_for_update": 0,
    "slow_device_start_all": 0,
    "link_manual_mode": True,
    "telnet_console_command": DEFAULT_TELNET_CONSOLE_COMMAND,
    "serial_console_command": DEFAULT_SERIAL_CONSOLE_COMMAND,
    "auto_close_console": True,
    "bring_console_to_front": True,
    "delay_console_all": 500,
    "default_local_news": False,
    "hide_news_dock_widget": False,
    "debug_level": 0,
}

GENERAL_SETTING_TYPES = {
    "style": str,
    "auto_launch_project_dialog": bool,
    "auto_screenshot": bool,
    "check_for_update": bool,
    "last_check_for_update": int,
    "slow_device_start_all": int,
    "link_manual_mode": bool,
    "telnet_console_command": str,
    "serial_console_command": str,
    "auto_close_console": bool,
    "bring_console_to_front": bool,
    "delay_console_all": int,
    "default_local_news": bool,
    "hide_news_dock_widget": bool,
    "debug_level": int,
}

GRAPHICS_VIEW_SETTINGS = {
    "scene_width": 2000,
    "scene_height": 1000,
    "draw_rectangle_selected_item": False,
    "draw_link_status_points": True,
    "default_label_font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
    "default_label_color": "#000000",
}

GRAPHICS_VIEW_SETTING_TYPES = {
    "scene_width": int,
    "scene_height": int,
    "draw_rectangle_selected_item": bool,
    "draw_link_status_points": bool,
    "default_label_font": str,
    "default_label_color": str,
}

LOCAL_SERVER_SETTINGS = {
    "path": "",
    "host": DEFAULT_LOCAL_SERVER_HOST,
    "port": DEFAULT_LOCAL_SERVER_PORT,
    "images_path": DEFAULT_IMAGES_PATH,
    "projects_path": DEFAULT_PROJECTS_PATH,
    "report_errors": True,
    "auto_start": True,
    "allow_console_from_anywhere": False,
    "console_start_port_range": 2001,
    "console_end_port_range": 5000,
    "udp_start_port_range": 10000,
    "udp_end_port_range": 20000,
}

LOCAL_SERVER_SETTING_TYPES = {
    "path": str,
    "host": str,
    "port": int,
    "images_path": str,
    "projects_path": str,
    "report_errors": bool,
    "auto_start": bool,
    "allow_console_from_anywhere": bool,
    "console_start_port_range": int,
    "console_end_port_range": int,
    "udp_start_port_range": int,
    "udp_end_port_range": int,
}

PACKET_CAPTURE_SETTINGS = {
    "packet_capture_reader_command": DEFAULT_PACKET_CAPTURE_READER_COMMAND,
    "command_auto_start": True,
    "packet_capture_analyzer_command": DEFAULT_PACKET_CAPTURE_ANALYZER_COMMAND,
}

PACKET_CAPTURE_SETTING_TYPES = {
    "packet_capture_reader_command": str,
    "command_auto_start": bool,
    "packet_capture_analyzer_command": str,
}

ENABLE_CLOUD = False

CLOUD_SETTINGS = {
    "cloud_user_name": "",
    "cloud_api_key": "",
    "cloud_store_api_key": False,
    # no default value at startup, users must choose and we need to know if they've already done it
    "cloud_store_api_key_chosen": False,
    "cloud_provider": "rackspace",
    "cloud_region": "",
    "instances_per_project": 0,
    "default_flavor": "",
    "new_instance_flavor": "",
    "accepted_terms": False,
    "instance_timeout": 30,
    "default_image": "",
}

CLOUD_SETTINGS_TYPES = {
    "cloud_user_name": str,
    "cloud_api_key": str,
    "cloud_store_api_key": bool,
    "cloud_store_api_key_chosen": bool,
    "cloud_provider": str,
    "cloud_region": str,
    "instances_per_project": int,
    "default_flavor": str,
    "new_instance_flavor": str,
    "accepted_terms": bool,
    "instance_timeout": int,
    "default_image": str,
}

# TODO proof of concept, needs review
CLOUD_PROVIDERS = {
    "rackspace": ("Rackspace", 'gns3.cloud.rackspace_ctrl.RackspaceCtrl'),
}

# heartbeat_freq is in milliseconds
DEFAULT_HEARTBEAT_FREQ = 60000



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/jseutterlst/gns3/gns3-gui/gns3/ui/cloud_preferences_page.ui'
#
# Created: Thu Jul 17 14:17:33 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_CloudPreferencesPageWidget(object):

    def setupUi(self, CloudPreferencesPageWidget):
        CloudPreferencesPageWidget.setObjectName(_fromUtf8("CloudPreferencesPageWidget"))
        CloudPreferencesPageWidget.resize(482, 485)
        self.gridLayout = QtGui.QGridLayout(CloudPreferencesPageWidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiImageTemplateLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiImageTemplateLabel.setObjectName(_fromUtf8("uiImageTemplateLabel"))
        self.gridLayout.addWidget(self.uiImageTemplateLabel, 16, 0, 1, 3)
        self.uiImageTemplateComboBox = QtGui.QComboBox(CloudPreferencesPageWidget)
        self.uiImageTemplateComboBox.setObjectName(_fromUtf8("uiImageTemplateComboBox"))
        self.gridLayout.addWidget(self.uiImageTemplateComboBox, 17, 0, 1, 1)
        self.uiNewInstancesLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiNewInstancesLabel.setObjectName(_fromUtf8("uiNewInstancesLabel"))
        self.gridLayout.addWidget(self.uiNewInstancesLabel, 9, 0, 1, 3)
        self.uiCreateAccountLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiCreateAccountLabel.setTextFormat(QtCore.Qt.RichText)
        self.uiCreateAccountLabel.setObjectName(_fromUtf8("uiCreateAccountLabel"))
        self.gridLayout.addWidget(self.uiCreateAccountLabel, 2, 2, 1, 1)
        self.uiUserNameLineEdit = QtGui.QLineEdit(CloudPreferencesPageWidget)
        self.uiUserNameLineEdit.setObjectName(_fromUtf8("uiUserNameLineEdit"))
        self.gridLayout.addWidget(self.uiUserNameLineEdit, 1, 0, 1, 1)
        self.uiAPIKeyLineEdit = QtGui.QLineEdit(CloudPreferencesPageWidget)
        self.uiAPIKeyLineEdit.setObjectName(_fromUtf8("uiAPIKeyLineEdit"))
        self.gridLayout.addWidget(self.uiAPIKeyLineEdit, 1, 2, 1, 1)
        self.uiStartNewProjectLayout = QtGui.QHBoxLayout()
        self.uiStartNewProjectLayout.setObjectName(_fromUtf8("uiStartNewProjectLayout"))
        self.uiNumOfInstancesSpinBox = QtGui.QSpinBox(CloudPreferencesPageWidget)
        self.uiNumOfInstancesSpinBox.setObjectName(_fromUtf8("uiNumOfInstancesSpinBox"))
        self.uiStartNewProjectLayout.addWidget(self.uiNumOfInstancesSpinBox)
        self.uiNumOfInstancesLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiNumOfInstancesLabel.setObjectName(_fromUtf8("uiNumOfInstancesLabel"))
        self.uiStartNewProjectLayout.addWidget(self.uiNumOfInstancesLabel)
        self.uiInstanceFlavorComboBox = QtGui.QComboBox(CloudPreferencesPageWidget)
        self.uiInstanceFlavorComboBox.setObjectName(_fromUtf8("uiInstanceFlavorComboBox"))
        self.uiStartNewProjectLayout.addWidget(self.uiInstanceFlavorComboBox)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.uiStartNewProjectLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.uiStartNewProjectLayout, 7, 0, 1, 3)
        self.uiTimeoutLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiTimeoutLabel.setWordWrap(True)
        self.uiTimeoutLabel.setObjectName(_fromUtf8("uiTimeoutLabel"))
        self.gridLayout.addWidget(self.uiTimeoutLabel, 12, 0, 1, 2)
        self.uiNewInstancesLayout = QtGui.QHBoxLayout()
        self.uiNewInstancesLayout.setObjectName(_fromUtf8("uiNewInstancesLayout"))
        self.uiNewInstanceFlavorComboBox = QtGui.QComboBox(CloudPreferencesPageWidget)
        self.uiNewInstanceFlavorComboBox.setObjectName(_fromUtf8("uiNewInstanceFlavorComboBox"))
        self.uiNewInstancesLayout.addWidget(self.uiNewInstanceFlavorComboBox)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.uiNewInstancesLayout.addItem(spacerItem1)
        self.uiTermsCheckBox = QtGui.QCheckBox(CloudPreferencesPageWidget)
        self.uiTermsCheckBox.setText(_fromUtf8(""))
        self.uiTermsCheckBox.setObjectName(_fromUtf8("uiTermsCheckBox"))
        self.uiNewInstancesLayout.addWidget(self.uiTermsCheckBox)
        self.uiTermsLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiTermsLabel.setTextFormat(QtCore.Qt.RichText)
        self.uiTermsLabel.setObjectName(_fromUtf8("uiTermsLabel"))
        self.uiNewInstancesLayout.addWidget(self.uiTermsLabel)
        self.gridLayout.addLayout(self.uiNewInstancesLayout, 10, 0, 2, 3)
        self.uiCloudProviderComboBox = QtGui.QComboBox(CloudPreferencesPageWidget)
        self.uiCloudProviderComboBox.setObjectName(_fromUtf8("uiCloudProviderComboBox"))
        self.gridLayout.addWidget(self.uiCloudProviderComboBox, 3, 0, 1, 1)
        self.uiUserNameLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiUserNameLabel.setObjectName(_fromUtf8("uiUserNameLabel"))
        self.gridLayout.addWidget(self.uiUserNameLabel, 0, 0, 1, 1)
        self.uiAPIKeyLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiAPIKeyLabel.setObjectName(_fromUtf8("uiAPIKeyLabel"))
        self.gridLayout.addWidget(self.uiAPIKeyLabel, 0, 2, 1, 1)
        self.uiRememberAPIKeyRadioButton = QtGui.QRadioButton(CloudPreferencesPageWidget)
        self.uiRememberAPIKeyRadioButton.setObjectName(_fromUtf8("uiRememberAPIKeyRadioButton"))
        self.gridLayout.addWidget(self.uiRememberAPIKeyRadioButton, 3, 2, 1, 1)
        self.uiStartNewProjectsLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiStartNewProjectsLabel.setObjectName(_fromUtf8("uiStartNewProjectsLabel"))
        self.gridLayout.addWidget(self.uiStartNewProjectsLabel, 6, 0, 1, 3)
        self.uiRegionComboBox = QtGui.QComboBox(CloudPreferencesPageWidget)
        self.uiRegionComboBox.setObjectName(_fromUtf8("uiRegionComboBox"))
        self.gridLayout.addWidget(self.uiRegionComboBox, 5, 0, 1, 1)
        self.uiCloudProviderLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiCloudProviderLabel.setObjectName(_fromUtf8("uiCloudProviderLabel"))
        self.gridLayout.addWidget(self.uiCloudProviderLabel, 2, 0, 1, 1)
        self.uiRegionLabel = QtGui.QLabel(CloudPreferencesPageWidget)
        self.uiRegionLabel.setObjectName(_fromUtf8("uiRegionLabel"))
        self.gridLayout.addWidget(self.uiRegionLabel, 4, 0, 1, 1)
        self.uiForgetAPIKeyRadioButton = QtGui.QRadioButton(CloudPreferencesPageWidget)
        self.uiForgetAPIKeyRadioButton.setObjectName(_fromUtf8("uiForgetAPIKeyRadioButton"))
        self.gridLayout.addWidget(self.uiForgetAPIKeyRadioButton, 4, 2, 1, 1)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.uiTimeoutSpinBox = QtGui.QSpinBox(CloudPreferencesPageWidget)
        self.uiTimeoutSpinBox.setMaximum(999)
        self.uiTimeoutSpinBox.setProperty("value", 30)
        self.uiTimeoutSpinBox.setObjectName(_fromUtf8("uiTimeoutSpinBox"))
        self.horizontalLayout_3.addWidget(self.uiTimeoutSpinBox)
        self.uiTimeoutLabel2 = QtGui.QLabel(CloudPreferencesPageWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTimeoutLabel2.sizePolicy().hasHeightForWidth())
        self.uiTimeoutLabel2.setSizePolicy(sizePolicy)
        self.uiTimeoutLabel2.setObjectName(_fromUtf8("uiTimeoutLabel2"))
        self.horizontalLayout_3.addWidget(self.uiTimeoutLabel2)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.gridLayout.addLayout(self.horizontalLayout_3, 14, 0, 2, 3)
        spacerItem3 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem3, 18, 0, 1, 1)

        self.retranslateUi(CloudPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(CloudPreferencesPageWidget)

    def retranslateUi(self, CloudPreferencesPageWidget):
        CloudPreferencesPageWidget.setWindowTitle(_translate("CloudPreferencesPageWidget", "Cloud", None))
        self.uiImageTemplateLabel.setText(_translate("CloudPreferencesPageWidget", "New instances use the following template:", None))
        self.uiNewInstancesLabel.setText(_translate("CloudPreferencesPageWidget", "New instances are:", None))
        self.uiCreateAccountLabel.setText(_translate("CloudPreferencesPageWidget", "No API Key? <a href=\"#\">Create Cloud Account.</a>", None))
        self.uiNumOfInstancesLabel.setText(_translate("CloudPreferencesPageWidget", "instance(s) with", None))
        self.uiTimeoutLabel.setText(_translate("CloudPreferencesPageWidget", "Instances are deleted after", None))
        self.uiTermsLabel.setText(_translate("CloudPreferencesPageWidget", "Accept Terms and Conditions", None))
        self.uiUserNameLabel.setText(_translate("CloudPreferencesPageWidget", "User Name:", None))
        self.uiAPIKeyLabel.setText(_translate("CloudPreferencesPageWidget", "API Key", None))
        self.uiRememberAPIKeyRadioButton.setText(_translate("CloudPreferencesPageWidget", "Remeber these settings\n"
                                                            "(Suggested for private computers)", None))
        self.uiStartNewProjectsLabel.setText(_translate("CloudPreferencesPageWidget", "Start new projects with:", None))
        self.uiCloudProviderLabel.setText(_translate("CloudPreferencesPageWidget", "Cloud provider", None))
        self.uiRegionLabel.setText(_translate("CloudPreferencesPageWidget", "Region (optional)", None))
        self.uiForgetAPIKeyRadioButton.setText(_translate("CloudPreferencesPageWidget", "Forget these settings on exit\n"
                                                          "(Suggested for public computers)", None))
        self.uiTimeoutLabel2.setText(_translate("CloudPreferencesPageWidget", "minutes of lost communication", None))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/getting_started_dialog.ui'
#
# Created: Thu Oct 16 19:58:05 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_GettingStartedDialog(object):

    def setupUi(self, GettingStartedDialog):
        GettingStartedDialog.setObjectName(_fromUtf8("GettingStartedDialog"))
        GettingStartedDialog.resize(778, 593)
        GettingStartedDialog.setModal(True)
        self.gridLayout = QtGui.QGridLayout(GettingStartedDialog)
        self.gridLayout.setSizeConstraint(QtGui.QLayout.SetFixedSize)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiWebView = QtWebKit.QWebView(GettingStartedDialog)
        self.uiWebView.setMinimumSize(QtCore.QSize(760, 540))
        self.uiWebView.setMaximumSize(QtCore.QSize(760, 540))
        self.uiWebView.setProperty("url", QtCore.QUrl(_fromUtf8("about:blank")))
        self.uiWebView.setObjectName(_fromUtf8("uiWebView"))
        self.gridLayout.addWidget(self.uiWebView, 0, 0, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.uiCheckBox = QtGui.QCheckBox(GettingStartedDialog)
        self.uiCheckBox.setChecked(True)
        self.uiCheckBox.setObjectName(_fromUtf8("uiCheckBox"))
        self.horizontalLayout.addWidget(self.uiCheckBox)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.buttonBox = QtGui.QDialogButtonBox(GettingStartedDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.horizontalLayout.addWidget(self.buttonBox)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(GettingStartedDialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), GettingStartedDialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), GettingStartedDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(GettingStartedDialog)

    def retranslateUi(self, GettingStartedDialog):
        GettingStartedDialog.setWindowTitle(_translate("GettingStartedDialog", "Getting started", None))
        self.uiCheckBox.setText(_translate("GettingStartedDialog", "Don\'t show this again", None))

from PyQt4 import QtWebKit



import socket
import select
import socketserver
import threading
import logging

log = logging.getLogger(__name__)

debug = True

if logging.getLogger().getEffectiveLevel() < 20:
    enable_debug = True

if debug:
    enable_debug = True

if enable_debug:
    log_format = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    log_level = logging.DEBUG
    log.setLevel(log_level)

    log_console = logging.StreamHandler()
    log_console.setFormatter(log_format)
    log_console.setLevel(log_level)
    log.addHandler(log_console)
    log.debug("DEBUG IS ENABLED")


class ThreadedTCPServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
    allow_reuse_address = True


class ThreadedTCPRequestHandler(socketserver.BaseRequestHandler):

    def handle(self):
        try:
            chan = self.ssh_transport.open_channel('direct-tcpip',
                                                   self.remote_address,
                                                   self.request.getpeername())
        except Exception as e:
            log.critical('Incoming request to %s:%s failed: %s' % (
                self.remote_address,
                repr(e)
            )
            )
            return

        if chan is None:
            log.critical('Incoming request to %s:%s was rejected by the SSH server.' %
                         (self.remote_address))
            return

        log.debug('Connected!  Tunnel open %r -> %r -> %r' % (self.request.getpeername(),
                                                              chan.getpeername(), self.remote_address))

        while True:
            r, w, x = select.select([self.request, chan], [], [])
            if self.request in r:
                data = self.request.recv(1024)
                if len(data) == 0:
                    break
                chan.send(data)
            if chan in r:
                data = chan.recv(1024)
                if len(data) == 0:
                    break
                self.request.send(data)

        peername = self.request.getpeername()
        chan.close()
        self.request.close()
        log.debug('Tunnel closed from %r' % (peername,))


class Endpoint(object):

    def __init__(self, local_address, remote_address, transport):
        """
        Store local and remote tunnel address information in the format:
        (ip, port) format.
        """

        self.local_address = local_address
        self.remote_address = remote_address
        self.transport = transport
        self.thread = None
        self.server = None

    def get(self):
        return (self.local_address, self.remote_address)

    def log_msg(self, msg):
        if self.thread:
            thread_name = self.thread.name
        else:
            thread_name = "Creating ID"

        log.info("%s: local %s:%s for remote %s:%s - %s" % (
            thread_name,
            self.local_address[0],
            self.local_address[1],
            self.remote_address[0],
            self.remote_address[1],
            msg,
        ))

    def _enable(self, local_address, remote_address, ssh_transport):
        # https://github.com/paramiko/paramiko/blob/master/demos/forward.py
        # This is a little convoluted, but lets me configure things for the Handler
        # object.  (SocketServer doesn't give Handlers any way to access the outer
        # server normally.)
        class EndPointHandler(ThreadedTCPRequestHandler):
            remote_address = self.remote_address
            local_address = self.local_address
            ssh_transport = self.transport

        server = ThreadedTCPServer(self.local_address, EndPointHandler)

        # https://docs.python.org/3.4/library/socketserver.html
        # Start a thread with the server -- that thread will then start one
        # more thread for each request
        server_thread = threading.Thread(target=server.serve_forever)
        # Exit the server thread when the main thread terminates
        server_thread.daemon = True
        server_thread.start()
        self.thread = server_thread
        self.server = server
        self.log_msg("Server thread running")

    def getId(self):
        return self.thread.name

    def enable(self):
        self.log_msg("Starting server thread")
        self._enable(self.local_address, self.remote_address, self.transport)

    def disable(self):
        if self.server:
            self.log_msg("Stopping server thread")
            self.server.shutdown()
        else:
            self.log_msg("No server thread running to stop")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/workspace/git/gns3-gui/gns3/modules/dynamips/ui/ethernet_hub_configuration_page.ui'
#
# Created: Sun Mar 16 11:16:57 2014
#      by: PyQt4 UI code generator 4.10
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_ethernetHubConfigPageWidget(object):

    def setupUi(self, ethernetHubConfigPageWidget):
        ethernetHubConfigPageWidget.setObjectName(_fromUtf8("ethernetHubConfigPageWidget"))
        ethernetHubConfigPageWidget.resize(381, 270)
        self.gridlayout = QtGui.QGridLayout(ethernetHubConfigPageWidget)
        self.gridlayout.setObjectName(_fromUtf8("gridlayout"))
        self.uiSettingsGroupBox = QtGui.QGroupBox(ethernetHubConfigPageWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSettingsGroupBox.sizePolicy().hasHeightForWidth())
        self.uiSettingsGroupBox.setSizePolicy(sizePolicy)
        self.uiSettingsGroupBox.setObjectName(_fromUtf8("uiSettingsGroupBox"))
        self.gridLayout = QtGui.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiNameLabel = QtGui.QLabel(self.uiSettingsGroupBox)
        self.uiNameLabel.setObjectName(_fromUtf8("uiNameLabel"))
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtGui.QLineEdit(self.uiSettingsGroupBox)
        self.uiNameLineEdit.setObjectName(_fromUtf8("uiNameLineEdit"))
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiPortsLabel = QtGui.QLabel(self.uiSettingsGroupBox)
        self.uiPortsLabel.setObjectName(_fromUtf8("uiPortsLabel"))
        self.gridLayout.addWidget(self.uiPortsLabel, 1, 0, 1, 1)
        self.uiPortsSpinBox = QtGui.QSpinBox(self.uiSettingsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPortsSpinBox.sizePolicy().hasHeightForWidth())
        self.uiPortsSpinBox.setSizePolicy(sizePolicy)
        self.uiPortsSpinBox.setMinimum(0)
        self.uiPortsSpinBox.setMaximum(65535)
        self.uiPortsSpinBox.setProperty("value", 1)
        self.uiPortsSpinBox.setObjectName(_fromUtf8("uiPortsSpinBox"))
        self.gridLayout.addWidget(self.uiPortsSpinBox, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(20, 71, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)
        self.gridlayout.addWidget(self.uiSettingsGroupBox, 0, 1, 1, 1)

        self.retranslateUi(ethernetHubConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(ethernetHubConfigPageWidget)

    def retranslateUi(self, ethernetHubConfigPageWidget):
        ethernetHubConfigPageWidget.setWindowTitle(_translate("ethernetHubConfigPageWidget", "Ethernet hub", None))
        self.uiSettingsGroupBox.setTitle(_translate("ethernetHubConfigPageWidget", "Settings", None))
        self.uiNameLabel.setText(_translate("ethernetHubConfigPageWidget", "Name:", None))
        self.uiPortsLabel.setText(_translate("ethernetHubConfigPageWidget", "Number of ports:", None))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Dialog to manage the snapshots.
"""

import shutil
import re
import time
import os

from ..qt import QtCore, QtGui
from ..utils.progress_dialog import ProgressDialog
from ..utils.process_files_thread import ProcessFilesThread
from ..ui.snapshots_dialog_ui import Ui_SnapshotsDialog
from ..topology import Topology
from ..node import Node


class SnapshotsDialog(QtGui.QDialog, Ui_SnapshotsDialog):

    """
    Snapshots dialog implementation.

    :param parent: parent widget
    """

    def __init__(self, parent, project_path, project_files_dir):

        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self._project_path = project_path
        self._project_files_dir = os.path.join(project_files_dir, "project-files")

        self.uiCreatePushButton.clicked.connect(self._createSnapshotSlot)
        self.uiDeletePushButton.clicked.connect(self._deleteSnapshotSlot)
        self.uiRestorePushButton.clicked.connect(self._restoreSnapshotSlot)
        self.uiSnapshotsList.itemDoubleClicked.connect(self._snapshotDoubleClickedSlot)
        self._listSnaphosts()

    def _listSnaphosts(self):
        """
        Lists all available snapshots.
        """

        self.uiSnapshotsList.clear()
        snapshot_dir = os.path.join(self._project_files_dir, "snapshots")
        if not os.path.isdir(snapshot_dir):
            return

        for snapshot in os.listdir(snapshot_dir):
            match = re.search(r"^(.*)_([0-9]+)_([0-9]+)", snapshot)
            if match:
                snapshot_name = match.group(1)
                snapshot_date = match.group(2)[:2] + '/' + match.group(2)[2:4] + '/' + match.group(2)[4:]
                snapshot_time = match.group(3)[:2] + ':' + match.group(3)[2:4] + ':' + match.group(3)[4:]
                item = QtGui.QListWidgetItem(self.uiSnapshotsList)
                item.setText("{} on {} at {}".format(snapshot_name, snapshot_date, snapshot_time))
                item.setData(QtCore.Qt.UserRole, os.path.join(snapshot_dir, snapshot))

        self.uiSnapshotsList.sortItems(QtCore.Qt.AscendingOrder)

        if self.uiSnapshotsList.count():
            self.uiSnapshotsList.setCurrentRow(0)
            self.uiDeletePushButton.setEnabled(True)
            self.uiRestorePushButton.setEnabled(True)
        else:
            self.uiDeletePushButton.setEnabled(False)
            self.uiRestorePushButton.setEnabled(False)

    def _createSnapshotSlot(self):
        """
        Slot to create a snapshot.
        """

        snapshot_name, ok = QtGui.QInputDialog.getText(self, "Snapshot", "Snapshot name:", QtGui.QLineEdit.Normal, "Unnamed")
        if ok and snapshot_name:
            from ..main_window import MainWindow
            MainWindow.instance().saveProject(self._project_path)
            snapshot_name = "{name}_{date}".format(name=snapshot_name, date=time.strftime("%d%m%y_%H%M%S"))
            snapshot_dir = os.path.join(self._project_files_dir, "snapshots", snapshot_name)
            thread = ProcessFilesThread(os.path.dirname(self._project_path), snapshot_dir, skip_dirs=["snapshots"])
            progress_dialog = ProgressDialog(thread, "Creating snapshot", "Copying project files...", "Cancel", parent=self)
            progress_dialog.show()
            progress_dialog.exec_()
            thread.deleteLater()
            self._listSnaphosts()

    def _deleteSnapshotSlot(self):
        """
        Slot to delete a snapshot.
        """

        item = self.uiSnapshotsList.currentItem()
        if item:
            snapshot_path = item.data(QtCore.Qt.UserRole)
            shutil.rmtree(snapshot_path, ignore_errors=True)
            self._listSnaphosts()

    def _restoreSnapshotSlot(self):
        """
        Slot to restore a snapshot.
        """

        item = self.uiSnapshotsList.currentItem()
        if item:
            snapshot_path = item.data(QtCore.Qt.UserRole)
            self._restoreSnapshot(snapshot_path)

    def _restoreSnapshot(self, snapshot_path):
        """
        Restores a snapshot.

        :param snapshot_path: path to the snapshot
        """

        match = re.search(r"^(.*)_([0-9]+)_([0-9]+)", os.path.basename(snapshot_path))
        if match:
            snapshot_name = match.group(1)
        else:
            snapshot_name = "Unknown"
        reply = QtGui.QMessageBox.question(self, "Snapshots", "This will discard any changes made to your project since the snapshot \"{}\" was taken?".format(snapshot_name),
                                           QtGui.QMessageBox.Ok, QtGui.QMessageBox.Cancel)
        if reply == QtGui.QMessageBox.Cancel:
            return

        # stop all the nodes
        topology = Topology.instance()
        for node in topology.nodes():
            if hasattr(node, "start") and node.status() == Node.started:
                node.stop()

        project_name, _ = os.path.splitext(os.path.basename(self._project_path))
        legacy_project_files_dir = os.path.join(snapshot_path, "{}-files".format(project_name))
        if os.path.exists(legacy_project_files_dir):
            # support for pre 1.3 snapshots
            for root, dirs, _ in os.walk(self._project_files_dir):
                dirs[:] = [d for d in dirs if d not in "snapshots"]
                for project_subdir in dirs:
                    project_subdir_path = os.path.join(root, project_subdir)
                    shutil.rmtree(project_subdir_path, ignore_errors=True)

            dirs = os.listdir(legacy_project_files_dir)
            for snapshot_subdir in dirs:
                snapshot_subdir_path = os.path.join(legacy_project_files_dir, snapshot_subdir)
                thread = ProcessFilesThread(snapshot_subdir_path, os.path.join(self._project_files_dir, snapshot_subdir))
                progress_dialog = ProgressDialog(thread, "Restoring snapshot", "Copying project files...", "Cancel", parent=self)
                progress_dialog.show()
                progress_dialog.exec_()
                thread.deleteLater()

            os.remove(self._project_path)
            shutil.copy(os.path.join(snapshot_path, os.path.basename(self._project_path)), self._project_path)

        else:
            thread = ProcessFilesThread(snapshot_path, os.path.dirname(self._project_path), skip_dirs=["snapshots"])
            progress_dialog = ProgressDialog(thread, "Restoring snapshot", "Copying project files...", "Cancel", parent=self)
            progress_dialog.show()
            progress_dialog.exec_()
            thread.deleteLater()

        from ..main_window import MainWindow
        MainWindow.instance().loadSnapshot(self._project_path)
        self.accept()

    def _snapshotDoubleClickedSlot(self, item):
        """
        Slot to restore a snapshot when it is double clicked.
        """

        snapshot_path = item.data(QtCore.Qt.UserRole)
        self._restoreSnapshot(snapshot_path)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for VirtualBox VMs.
"""

from gns3.qt import QtGui
from gns3.dialogs.node_configurator_dialog import ConfigurationError

from ..ui.virtualbox_vm_configuration_page_ui import Ui_virtualBoxVMConfigPageWidget


class VirtualBoxVMConfigurationPage(QtGui.QWidget, Ui_virtualBoxVMConfigPageWidget):

    """
    QWidget configuration page for VirtualBox VMs.
    """

    def __init__(self):

        QtGui.QWidget.__init__(self)
        self.setupUi(self)

        self.uiAdapterTypesComboBox.clear()
        self.uiAdapterTypesComboBox.addItems(["PCnet-PCI II (Am79C970A)",
                                              "PCNet-FAST III (Am79C973)",
                                              "Intel PRO/1000 MT Desktop (82540EM)",
                                              "Intel PRO/1000 T Server (82543GC)",
                                              "Intel PRO/1000 MT Server (82545EM)",
                                              "Paravirtualized Network (virtio-net)"])

        # TODO: finish VM name change
        self.uiVMListLabel.hide()
        self.uiVMListComboBox.hide()

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the VirtualBox VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        if not group:

            # set the device name
            if "name" in settings:
                self.uiNameLineEdit.setText(settings["name"])
            else:
                self.uiNameLabel.hide()
                self.uiNameLineEdit.hide()

            if "console" in settings:
                self.uiConsolePortSpinBox.setValue(settings["console"])
            else:
                self.uiConsolePortLabel.hide()
                self.uiConsolePortSpinBox.hide()

            if "linked_base" in settings:
                self.uiBaseVMCheckBox.setChecked(settings["linked_base"])
            else:
                self.uiBaseVMCheckBox.hide()

        else:
            self.uiNameLabel.hide()
            self.uiNameLineEdit.hide()
            self.uiConsolePortLabel.hide()
            self.uiConsolePortSpinBox.hide()
            self.uiVMListLabel.hide()
            self.uiVMListComboBox.hide()

        self.uiAdaptersSpinBox.setValue(settings["adapters"])
        index = self.uiAdapterTypesComboBox.findText(settings["adapter_type"])
        if index != -1:
            self.uiAdapterTypesComboBox.setCurrentIndex(index)
        self.uiUseAnyAdapterCheckBox.setChecked(settings["use_any_adapter"])
        self.uiVMRamSpinBox.setValue(settings["ram"])
        self.uiHeadlessModeCheckBox.setChecked(settings["headless"])
        self.uiEnableConsoleCheckBox.setChecked(settings["enable_remote_console"])


    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the VirtualBox VM settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of VMs
        """

        # these settings cannot be shared by nodes and updated
        # in the node configurator.
        if not group:

            if "name" in settings:
                name = self.uiNameLineEdit.text()
                if not name:
                    QtGui.QMessageBox.critical(self, "Name", "VirtualBox name cannot be empty!")
                else:
                    settings["name"] = name

            if "console" in settings:
                settings["console"] = self.uiConsolePortSpinBox.value()

            if "linked_base" in settings:
                settings["linked_base"] = self.uiBaseVMCheckBox.isChecked()

            settings["enable_remote_console"] = self.uiEnableConsoleCheckBox.isChecked()

        else:
            del settings["name"]
            del settings["console"]
            del settings["enable_remote_console"]

        settings["ram"] = self.uiVMRamSpinBox.value()
        settings["adapter_type"] = self.uiAdapterTypesComboBox.currentText()
        settings["headless"] = self.uiHeadlessModeCheckBox.isChecked()
        settings["use_any_adapter"] = self.uiUseAnyAdapterCheckBox.isChecked()

        adapters = self.uiAdaptersSpinBox.value()
        if node:
            if settings["adapters"] != adapters:
                # check if the adapters settings have changed
                node_ports = node.ports()
                for node_port in node_ports:
                    if not node_port.isFree():
                        QtGui.QMessageBox.critical(self, node.name(), "Changing the number of adapters while links are connected isn't supported yet! Please delete all the links first.")
                        raise ConfigurationError()
        settings["adapters"] = adapters



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Thread to repeatedly try to connect to a network resource.
"""

import socket
import time
from ..qt import QtCore


class WaitForConnectionThread(QtCore.QThread):

    """
    Thread to wait for a connection.

    :param host: destination host or IP address
    :param port: destination port
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    completed = QtCore.pyqtSignal()
    update = QtCore.pyqtSignal(int)

    def __init__(self, host, port):

        QtCore.QThread.__init__(self)
        self._is_running = False
        self._host = host
        self._port = port

    def run(self):
        """
        Thread starting point.
        """

        self._is_running = True
        connection_success = False
        begin = time.time()

        # try to connect for 30 seconds
        while time.time() - begin < 30.0:
            if not self._is_running:
                return
            time.sleep(0.01)
            sock = None
            try:
                sock = socket.create_connection((self._host, self._port), timeout=10)
            except OSError as e:
                last_exception = e
                continue
            finally:
                if sock:
                    sock.close()
            connection_success = True
            break

        if not self._is_running:
            return

        if not connection_success:

            # let the GUI know about the connection was unsuccessful and finish the thread
            self.error.emit("Could not connect to {} on port {}: {}".format(self._host,
                                                                            self._port,
                                                                            last_exception), True)
            return

        # connection has been successful, let's inform the GUI before the thread exits
        self.completed.emit()

    def stop(self):
        """
        Stops this thread as soon as possible.
        """

        self._is_running = False



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Keeps track of all cloud instances the app has started.
"""

from .qt import QtCore
from gns3.topology import TopologyInstance

import logging

log = logging.getLogger(__name__)


class CloudInstances(QtCore.QObject):

    """
    This class stores the instances that gns3 gui has started.  This can be different than the list
    of instances in the topology that can be changed when switching projects.  This list is not touched
    when switching projects and is stored in the .ini file.
    """

    def __init__(self, *args, **kwargs):
        super(CloudInstances, self).__init__(*args, **kwargs)
        self._instances = []

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of CloudInstances.

        :returns: instance of CloudInstances
        """

        if not hasattr(CloudInstances, "_instance"):
            CloudInstances._instance = CloudInstances()
        return CloudInstances._instance

    @property
    def instances(self):
        return self._instances

    def clear(self):
        self._instances.clear()

    def add(self, topology_instance):
        self._instances.append(topology_instance)

    def add_instance(self, instance, keypair):
        if instance is None:
            return
        existing = self.get_instance(instance.id)
        if existing is None:
            ti = TopologyInstance(instance.name, instance.id, instance.extra['flavorId'],
                                  instance.extra['imageId'], keypair.private_key, keypair.public_key)
            self._instances.append(ti)
            self.save()

    def update_instances(self, instances):
        """
        Compare with the existing list of instances to purge instances that no
        longer exist.
        """
        save_needed = False
        # Look for instances that have been deleted
        for stored in self._instances:
            found = False
            for dynamic in instances:
                if stored.id == dynamic.id:
                    found = True
                    break

            if not found:
                self._instances.remove(stored)
                save_needed = True

        if save_needed:
            self.save()

    def update_host_for_instance(self, host, instance_id):
        """
        Update the public IP for the instance.
        """
        for instance in self.instances:
            if instance.id == instance_id:
                if instance.host != host:
                    instance.host = host
                    self.save()

    def save(self):
        """
        Save the list of cloud instances to the config file
        """
        log.debug('Saving cloud instances')
        settings = QtCore.QSettings()
        settings.beginGroup("CloudInstances")
        settings.remove("")

        # Save the instances
        settings.beginWriteArray("cloud_instance", len(self._instances))
        index = 0
        for instance in self._instances:
            settings.setArrayIndex(index)
            for name in instance.fields():
                value = getattr(instance, name) if not None else ""
                log.debug('{}={}'.format(name, str(value)[0:60]))
                settings.setValue(name, value)
            index += 1
        settings.endArray()
        settings.endGroup()

    def load(self):
        """
        Load instance info from the config file to the topology
        """
        log.debug('Loading cloud instances')
        settings = QtCore.QSettings()
        settings.beginGroup("CloudInstances")

        # Load the instances
        size = settings.beginReadArray("cloud_instance")
        for index in range(0, size):
            settings.setArrayIndex(index)
            info = {}
            for name in TopologyInstance.fields():
                value = settings.value(name, "")
                log.debug('{}={}'.format(name, str(value)[0:60]))
                info[name] = value
            ti = TopologyInstance(**info)
            self._instances.append(ti)

    def get_instance(self, instance_id):
        """
        Retrieve a TopologyInstance objects if present
        """
        for i in self._instances:
            if i.id == instance_id:
                return i
        return None



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for port objects.
"""

import os
import sys
import tempfile
import shutil
import subprocess
import shlex
import logging

log = logging.getLogger(__name__)

from gns3.utils.normalize_filename import normalize_filename
from ..qt import QtCore
from ..local_config import LocalConfig
from ..nios.nio_udp import NIOUDP
from ..settings import PACKET_CAPTURE_SETTINGS, PACKET_CAPTURE_SETTING_TYPES


class Port(object):

    """
    Base port.

    :param name: port name (string)
    :param default_nio: NIO object to use by default
    :param stub: indicates a stub port
    """

    _instance_count = 1
    _settings = {}

    # port statuses
    stopped = 0
    started = 1
    suspended = 2

    def __init__(self, name, default_nio=None, stub=False):

        # create an unique ID
        self._id = Port._instance_count
        Port._instance_count += 1

        self._name = name
        self._short_name = None
        self._port_number = None
        self._adapter_number = None
        self._stub = stub
        self._link_id = None
        self._port_label = None
        self._status = Port.stopped
        self._data = {}
        self._destination_node = None
        self._destination_port = None

        self._capture_supported = False
        self._capture_file_path = ""
        self._capturing = False
        self._tail_process = None
        self._capture_reader_process = None
        self._capture_analyzer_process = None

        if default_nio is None:
            self._default_nio = NIOUDP
        else:
            self._default_nio = default_nio
        self._nio = None

    def id(self):
        """
        Returns an unique identifier for this port.

        :returns: port identifier (integer)
        """

        return self._id

    def setId(self, new_id):
        """
        Sets an identifier for this port.

        :param new_id: node identifier (integer)
        """

        self._id = new_id

        # update the instance count to avoid conflicts
        if new_id >= Port._instance_count:
            Port._instance_count = new_id + 1

    @classmethod
    def reset(cls):
        """
        Reset the instance count.
        """

        cls._instance_count = 1

    def name(self):
        """
        Returns the name of this port.

        :returns: current port name (string)
        """

        return self._name

    def setName(self, new_name):
        """
        Sets a new name for this port.

        :param new_name: new port name (string)
        """

        self._name = new_name

    def shortName(self):
        """
        Returns the short name of this port.

        :returns: current short port name (string)
        """

        if not self._short_name:
            return self._name
        return self._short_name

    def setShortName(self, short_name):
        """
        Sets a new short name for this port.

        :param short_name: short port name (string)
        """

        self._short_name = short_name

    def status(self):
        """
        Returns the status of this port.
        0 = stopped, 1 = started, 2 = suspended.

        :returns: port status (integer)
        """

        return self._status

    def setStatus(self, status):
        """
        Sets a status for this port.
        0 = stopped, 1 = started, 2 = suspended.

        :param status: port status (integer)
        """

        self._status = status

    def adapterNumber(self):
        """
        Returns the slot number for this port.

        :returns: current slot number (integer)
        """

        return self._adapter_number

    def setAdapterNumber(self, adapter_number):
        """
        Sets the adapter number for this port.

        :param adapter_number: new slot number (integer)
        """

        self._adapter_number = adapter_number

    def portNumber(self):
        """
        Returns the port number for this port.

        :returns: current port number (integer)
        """

        return self._port_number

    def setPortNumber(self, port_number):
        """
        Sets the port number for this port.

        :param port: new port number (integer)
        """

        self._port_number = port_number

    def destinationNode(self):
        """
        Returns the destination node

        :returns: destination Node instance
        """

        return self._destination_node

    def setDestinationNode(self, node):
        """
        Sets a new destination Node instance for this port.

        :param node: new destination Node instance
        """

        self._destination_node = node

    def destinationPort(self):
        """
        Returns the destination Port instance

        :returns: destination Port instance
        """

        return self._destination_port

    def setDestinationPort(self, port):
        """
        Sets a new destination Port instance for this port.

        :param port: new destination Port instance
        """

        self._destination_port = port

    def defaultNio(self):
        """
        Returns the default NIO for this port.

        :returns: NIO object
        """

        return self._default_nio

    def nio(self):
        """
        Returns the NIO attached to this port.

        :returns: NIO instance
        """

        return self._nio

    def setNio(self, nio):
        """
        Attach a NIO to this port.

        :param nio: NIO instance
        """

        self._nio = nio

    def linkId(self):
        """
        Returns the link id connected to this port.

        :returns: link id (integer)
        """

        return self._link_id

    def setLinkId(self, link_id):
        """
        Adds the link id connected to this port.

        :param link_id: link id (integer)
        """

        self._link_id = link_id

    def description(self, short=False):
        """
        Returns the text description of this port.

        :param short: returns a shorter description.

        :returns: description
        """

        if self._destination_node and self._destination_port:
            if short:
                return "<-> {port} {name}".format(port=self._destination_port.shortName(),
                                                  name=self._destination_node.name())
            return "connected to {name} on port {port}".format(name=self._destination_node.name(),
                                                               port=self._destination_port.name())
        return ""

    def setFree(self):
        """
        Frees this port.
        """

        self._nio = None
        self._link_id = None
        self._destination_node = None
        self._destination_port = None
        self._port_label = None
        self.stopPacketCapture()

    def isFree(self):
        """
        Checks if this port is free to use (no NIO attached).

        :returns: boolean
        """

        if self._nio:
            return False
        return True

    def isStub(self):
        """
        Checks if this is a stub port.

        :returns: boolean
        """

        return self._stub

    @staticmethod
    def linkType():
        """
        Default link type to be used.

        :returns: string
        """

        return "Ethernet"

    @staticmethod
    def dataLinkTypes():
        """
        Returns the supported PCAP DLTs.

        :return: dictionary
        """

        return {"Ethernet": "DLT_EN10MB"}

    def data(self):
        """
        Returns the data associated with this port.

        :returns: current port data (dictionary)
        """

        return self._data

    def setData(self, new_data):
        """
        Sets data to be associated with this port.

        :param new_data: new port data (dictionary)
        """

        self._data = new_data

    def label(self):
        """
        Returns the port label.

        :return: NoteItem instance.
        """

        return self._port_label

    def setLabel(self, label):
        """
        Sets a port label.

        :param label: NoteItem instance.
        """

        self._port_label = label

    @classmethod
    def loadPacketCaptureSettings(cls):
        """
        Loads the packet capture settings from the persistent settings file.
        """

        local_config = LocalConfig.instance()

        # restore the packet capture settings from QSettings (for backward compatibility)
        legacy_settings = {}
        settings = QtCore.QSettings()
        settings.beginGroup("PacketCapture")
        for name in PACKET_CAPTURE_SETTINGS.keys():
            if settings.contains(name):
                legacy_settings[name] = settings.value(name, type=PACKET_CAPTURE_SETTING_TYPES[name])
        settings.remove("")
        settings.endGroup()
        if legacy_settings:
            local_config.saveSectionSettings("PacketCapture", legacy_settings)

        cls._settings = local_config.loadSectionSettings("PacketCapture", PACKET_CAPTURE_SETTINGS)

    @classmethod
    def setPacketCaptureSettings(cls, new_settings):
        """
        Sets new packet capture settings.

        :param new_settings: settings dictionary
        """

        cls._settings.update(new_settings)
        LocalConfig.instance().saveSectionSettings("PacketCapture", cls._settings)

    @classmethod
    def packetCaptureSettings(cls):
        """
        Returns the packet capture settings.

        :returns: settings dictionary
        """

        return cls._settings

    def setPacketCaptureSupported(self, value):
        """
        Sets either packet capture is support or not on this port

        :param value: boolean
        """

        self._capture_supported = value

    def packetCaptureSupported(self):
        """
        Returns either packet capture is support or not on this port

        :return: boolean
        """

        return self._capture_supported

    def capturing(self):
        """
        Returns either packet capture is active

        :return: boolean
        """

        return self._capturing

    def startPacketCapture(self, capture_file_path):
        """
        Starts a packet capture.

        :param capture_file_path: PCAP capture output file
        """

        self._capturing = True
        self._capture_file_path = capture_file_path
        log.info("Saving packet capture to {}".format(capture_file_path))
        if os.path.isfile(capture_file_path) and self._settings["command_auto_start"]:
            self.startPacketCaptureReader()

    def stopPacketCapture(self):
        """
        Stops a packet capture.
        """

        self._capturing = False
        self._capture_file_path = ""
        if self._tail_process and self._tail_process.poll() is None:
            self._tail_process.kill()
            self._tail_process = None
        self._capture_reader_process = None

    def startPacketCaptureReader(self):
        """
        Starts the packet capture reader.
        """

        if not os.path.isfile(self._capture_file_path):
            raise FileNotFoundError("the {} capture file does not exist on this host".format(self._capture_file_path))

        if self._tail_process and self._tail_process.poll() is None:
            self._tail_process.kill()
            self._tail_process = None
        if self._capture_reader_process and self._capture_reader_process.poll() is None:
            self._capture_reader_process.kill()
            self._capture_reader_process = None

        command = self._settings["packet_capture_reader_command"]
        command = command.replace("%c", '"' + self._capture_file_path + '"')

        if "|" in command:
            # live traffic capture (using tail)
            env = None
            command1, command2 = command.split("|", 1)
            info = None
            if sys.platform.startswith("win"):
                # hide tail window on Windows
                info = subprocess.STARTUPINFO()
                info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                info.wShowWindow = subprocess.SW_HIDE
                if hasattr(sys, "frozen"):
                    env = {"PATH": os.path.dirname(os.path.abspath(sys.executable))}  # for Popen to find tail.exe
                command1 = command1.strip()
                command2 = command2.strip()
            else:
                command1 = shlex.split(command1)
                command2 = shlex.split(command2)

            self._tail_process = subprocess.Popen(command1, startupinfo=info, stdout=subprocess.PIPE, env=env)
            self._capture_reader_process = subprocess.Popen(command2, stdin=self._tail_process.stdout, stdout=subprocess.PIPE)
            self._tail_process.stdout.close()
        else:
            # normal traffic capture
            if not sys.platform.startswith("win"):
                command = shlex.split(command)
            self._capture_reader_process = subprocess.Popen(command)

    def startPacketCaptureAnalyzer(self):
        """
        Starts the packet capture analyzer.
        """

        if not os.path.isfile(self._capture_file_path):
            raise FileNotFoundError("the {} capture file does not exist on this host".format(self._capture_file_path))

        if self._capture_analyzer_process and self._capture_analyzer_process.poll() is None:
            self._capture_analyzer_process.kill()
            self._capture_analyzer_process = None

        command = self._settings["packet_capture_analyzer_command"]
        temp_capture_file_path = os.path.join(tempfile.gettempdir(), os.path.basename(self._capture_file_path))

        try:
            shutil.copy(self._capture_file_path, temp_capture_file_path)
        except OSError:
            raise

        command = command.replace("%c", '"' + temp_capture_file_path + '"')
        self._capture_analyzer_process = subprocess.Popen(command)

    def captureFileName(self, source_node_name):
        """
        Returns a capture file name.

        :param source_node_name: source node name

        :return: capture file name
        """

        capture_file_name = "{}_{}_to_{}_{}".format(source_node_name,
                                                    self.name().replace('/', '-'),
                                                    self.destinationNode().name(),
                                                    self.destinationPort().name().replace('/', '-'))

        return normalize_filename(capture_file_name) + ".pcap"

    def dump(self):
        """
        Returns a representation of this port.

        :returns: dictionary
        """

        port = {"name": self._name,
                "id": self._id}

        if self._nio:
            port["nio"] = str(self._nio)
        if self._port_number is not None:
            port["port_number"] = self._port_number
        if self._adapter_number is not None:
            port["adapter_number"] = self._adapter_number
        if self._stub:
            port["stub"] = self._stub
        if self.description():
            port["description"] = self.description()
        if self._link_id is not None:
            port["link_id"] = self._link_id

        return port

    def __str__(self):

        return self._name



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Keeps track of all the local and remote servers and their settings.
"""

import sys
import os
import shlex
import signal
import shutil
import subprocess

from .qt import QtGui, QtCore, QtNetwork
from .http_client import HTTPClient
from .local_config import LocalConfig
from .settings import LOCAL_SERVER_SETTINGS, LOCAL_SERVER_SETTING_TYPES
from .local_server_config import LocalServerConfig
from collections import OrderedDict

import logging
log = logging.getLogger(__name__)


class Servers(QtCore.QObject):

    """
    Server management class.
    """

    # to let other pages know about remote server updates
    updated_signal = QtCore.Signal()

    def __init__(self):

        super(Servers, self).__init__()
        self._local_server = None
        self._remote_servers = {}
        self._cloud_servers = {}
        self._local_server_path = ""
        self._local_server_auto_start = True
        self._local_server_allow_console_from_anywhere = False
        self._local_server_proccess = None
        self._network_manager = QtNetwork.QNetworkAccessManager(self)
        self._local_server_settings = {}
        self._remote_server_iter_pos = 0
        self._loadSettings()

        host = self._local_server_settings["host"]
        port = self._local_server_settings["port"]
        url = "http://{host}:{port}".format(host=host, port=port)
        self._local_server = HTTPClient(url, self._network_manager)
        self._local_server.setLocal(True)
        log.info("New local server connection {} registered".format(url))

    @staticmethod
    def _findLocalServer(self):
        """
        Finds the local server path.

        :return: path to the local server
        """

        if sys.platform.startswith("win") and hasattr(sys, "frozen"):
            local_server_path = os.path.join(os.getcwd(), "gns3server.exe")
        elif sys.platform.startswith("darwin") and hasattr(sys, "frozen"):
            local_server_path = os.path.join(os.getcwd(), "server/Contents/MacOS/gns3server")
        else:
            local_server_path = shutil.which("gns3server")

        if local_server_path is None:
            return ""
        return local_server_path

    def _loadSettings(self):
        """
        Loads the server settings from the persistent settings file.
        """

        local_config = LocalConfig.instance()

        # restore the local server settings from QSettings (for backward compatibility)
        legacy_settings = {}
        settings = QtCore.QSettings()
        settings.beginGroup(self.__class__.__name__)
        for name in LOCAL_SERVER_SETTINGS.keys():
            if settings.contains(name):
                legacy_settings[name] = settings.value(name, type=LOCAL_SERVER_SETTING_TYPES[name])

        # load the remote servers
        size = settings.beginReadArray("remote")
        for index in range(0, size):
            settings.setArrayIndex(index)
            host = settings.value("host", "")
            port = settings.value("port", 0, type=int)
            if host and port:
                self._addRemoteServer(host, port)
        settings.endArray()
        settings.remove("")
        settings.endGroup()

        if legacy_settings:
            local_config.saveSectionSettings("LocalServer", legacy_settings)

        self._local_server_settings = local_config.loadSectionSettings("LocalServer", LOCAL_SERVER_SETTINGS)
        if not os.path.exists(self._local_server_settings["path"]):
            self._local_server_settings["path"] = self._findLocalServer(self)

        settings = local_config.settings()
        if "RemoteServers" in settings:
            for remote_server in settings["RemoteServers"]:
                self._addRemoteServer(remote_server["host"], remote_server["port"])

        # keep the config file sync
        self._saveSettings()
        return settings

    def _saveSettings(self):
        """
        Saves the server settings to a persistent settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings("LocalServer", self._local_server_settings)

        # save the remote servers
        remote_servers = []
        for server in self._remote_servers.values():
            remote_servers.append({"host": server.host,
                                   "port": server.port})
        LocalConfig.instance().setSettings({"RemoteServers": remote_servers})

        # save some settings to the local server config files
        server_settings = OrderedDict([
            ("host", self._local_server_settings["host"]),
            ("port", self._local_server_settings["port"]),
            ("images_path", self._local_server_settings["images_path"]),
            ("projects_path", self._local_server_settings["projects_path"]),
            ("console_start_port_range", self._local_server_settings["console_start_port_range"]),
            ("console_end_port_range", self._local_server_settings["console_end_port_range"]),
            ("udp_start_port_range", self._local_server_settings["udp_start_port_range"]),
            ("udp_start_end_range", self._local_server_settings["udp_end_port_range"]),
            ("report_errors", self._local_server_settings["report_errors"]),
        ])
        config = LocalServerConfig.instance()
        config.saveSettings("Server", server_settings)

        if self._local_server and self._local_server.connected():
            self._local_server.post("/config/reload", None)

    def localServerSettings(self):
        """
        Returns the local server settings.

        :returns: local server settings (dict)
        """

        return self._local_server_settings

    def setLocalServerSettings(self, settings):
        """
        Set new local server settings.

        :param settings: local server settings (dict)
        """

        if settings["host"] != self._local_server_settings["host"] or settings["port"] != self._local_server_settings["port"]:
            url = "http://{host}:{port}".format(host=settings["host"], port=settings["port"])
            self._local_server = HTTPClient(url, self._network_manager)
            self._local_server.setLocal(True)
            log.info("New local server connection {} registered".format(url))

        self._local_server_settings.update(settings)

    def localServerAutoStart(self):
        """
        Returns either the local server
        is automatically started on startup.

        :returns: boolean
        """

        return self._local_server_settings["auto_start"]

    def localServerPath(self):
        """
        Returns the local server path.

        :returns: path to local server program.
        """

        return self._local_server_settings["path"]

    def startLocalServer(self):
        """
        Starts the local server process.
        """

        path = self.localServerPath()
        host = self._local_server.host
        port = self._local_server.port
        command = '"{executable}" --host {host} --port {port} --local'.format(executable=path,
                                                                              host=host,
                                                                              port=port)

        if self._local_server_settings["allow_console_from_anywhere"]:
            # allow connections to console from remote addresses
            command += " --allow"

        log.info("Starting local server process with {}".format(command))
        try:
            if sys.platform.startswith("win"):
                # use the string on Windows
                self._local_server_proccess = subprocess.Popen(command, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP)
            else:
                # use arguments on other platforms
                args = shlex.split(command)
                self._local_server_proccess = subprocess.Popen(args)
        except (OSError, subprocess.SubprocessError) as e:
            log.warning('Could not start local server "{}": {}'.format(command, e))
            return False

        log.info("Local server process has started (PID={})".format(self._local_server_proccess.pid))
        return True

    def localServerIsRunning(self):
        """
        Returns either the local server is running.

        :returns: boolean
        """

        if self._local_server_proccess and self._local_server_proccess.poll() is None:
            return True
        return False

    def stopLocalServer(self, wait=False):
        """
        Stops the local server.

        :param wait: wait for the server to stop
        """

        if self.localServerIsRunning():
            log.info("Stopping local server (PID={})".format(self._local_server_proccess.pid))
            # local server is running, let's stop it
            if wait:
                try:
                    # wait for the server to stop for maximum 2 seconds
                    self._local_server_proccess.wait(timeout=2)
                except subprocess.TimeoutExpired:
                    # the local server couldn't be stopped with the normal procedure
                    if sys.platform.startswith("win"):
                        self._local_server_proccess.send_signal(signal.CTRL_BREAK_EVENT)
                    else:
                        self._local_server_proccess.send_signal(signal.SIGINT)
                    try:
                        # wait for the server to stop for maximum 2 seconds
                        self._local_server_proccess.wait(timeout=2)
                    except subprocess.TimeoutExpired:
                        from .main_window import MainWindow
                        main_window = MainWindow.instance()
                        proceed = QtGui.QMessageBox.question(main_window,
                                                             "Local server",
                                                             "The Local server cannot be stopped, would you like to kill it?",
                                                             QtGui.QMessageBox.Yes,
                                                             QtGui.QMessageBox.No)

                        if proceed == QtGui.QMessageBox.Yes:
                            self._local_server_proccess.kill()

    def localServer(self):
        """
        Returns the local server.

        :returns: Server instance
        """

        return self._local_server

    def _addRemoteServer(self, host, port):
        """
        Adds a new remote server.

        :param host: host or address of the server
        :param port: port of the server (integer)

        :returns: the new remote server
        """

        server_socket = "{host}:{port}".format(host=host, port=port)
        url = "http://{server_socket}".format(server_socket=server_socket)
        server = HTTPClient(url, self._network_manager)
        server.setLocal(False)
        self._remote_servers[server_socket] = server
        log.info("New remote server connection {} registered".format(url))
        return server

    def getRemoteServer(self, host, port):
        """
        Gets a remote server.

        :param host: host address
        :param port: port

        :returns: remote server (HTTPClient instance)
        """

        for server in self._remote_servers.values():
            if server.host == host and int(server.port) == int(port):
                return server

        return self._addRemoteServer(host, port)

    def getServerFromString(self, string):
        """
        Finds a server instance from its string representation.
        """

        if string == "local":
            return self._local_server

        (host, port) = string.split(":")
        return self.getRemoteServer(host, port)

    def updateRemoteServers(self, servers):
        """
        Updates the remote servers list.

        :param servers: servers dictionary.
        """

        for server_id, server in self._remote_servers.copy().items():
            if server_id not in servers:
                if server.connected():
                    server.close()
                log.info("Remote server connection {} unregistered".format(server.url()))
                del self._remote_servers[server_id]

        for server_id, server in servers.items():
            if server_id in self._remote_servers:
                continue

            host = server["host"]
            port = server["port"]
            url = "http://{host}:{port}".format(host=host, port=port)
            new_server = HTTPClient(url, self._network_manager)
            new_server.setLocal(False)
            self._remote_servers[server_id] = new_server
            log.info("New remote server connection {} registered".format(url))

        self.updated_signal.emit()

    def remoteServers(self):
        """
        Returns all the remote servers.

        :returns: remote servers dictionary
        """

        return self._remote_servers

    # def getCloudServer(self, host, port, ca_file, auth_user, auth_password, ssh_pkey, instance_id):
    #     """
    #     Return a websocket connection to the cloud server, creating one if none exists.
    #
    #     :param host: host ip address of the cloud server
    #     :param port: port the gns3server process is listening on
    #     :param ca_file: Path to the SSL cert that the server must present
    #     :returns: a websocket connection to the cloud server
    #     """
    #
    #     for server in self._remote_servers.values():
    #         if server.host == host and int(server.port) == int(port):
    #             return server
    #
    #     heartbeat_freq = self._settings.value("heartbeat_freq", DEFAULT_HEARTBEAT_FREQ)
    #     return self._addCloudServer(host, port, ca_file, auth_user, auth_password, ssh_pkey,
    #                                 heartbeat_freq, instance_id)
    #
    # def _addCloudServer(self, host, port, ca_file, auth_user, auth_password, ssh_pkey,
    #                     heartbeat_freq, instance_id):
    #     """
    #     Create a websocket connection to the specified cloud server
    #
    #     :param host: host ip address of the server
    #     :param port: port the gns3server process is listening on
    #     :param ca_file: Path to the SSL cert that the server must present
    #     :param heartbeat_freq: The interval to send heartbeats to the server
    #
    #     :returns: a websocket connection to the cloud server
    #     """
    #
    #     url = "wss://{host}:{port}".format(host=host, port=port)
    #     log.debug('Starting SecureWebSocketClient url={}'.format(url))
    #     log.debug('Starting SecureWebSocketClient ca_file={}'.format(ca_file))
    #     log.debug('Starting SecureWebSocketClient ssh_pkey={}'.format(ssh_pkey))
    #     server = SecureWebSocketClient(url, instance_id=instance_id)
    #     server.setSecureOptions(ca_file, auth_user, auth_password, ssh_pkey)
    #     server.setCloud(True)
    #     server.enableHeartbeatsAt(heartbeat_freq)
    #     self._cloud_servers[host] = server
    #     log.info("new remote server connection {} registered".format(url))
    #     return server
    #
    # def anyCloudServer(self):
    #     # Return the first server for now
    #     for key, value in self._cloud_servers.items():
    #         return value
    #     return None
    #
    # def cloudServerById(self, instance_id):
    #     """
    #     Return the server with the specified instance id, or None.
    #     """
    #     for cs in self.cloud_servers.values():
    #         if cs.instance_id == instance_id:
    #             return cs
    #     return None
    #
    # def removeCloudServer(self, server):
    #     try:
    #         cs = self.cloud_servers[server.host]
    #         cs.close_connection()
    #         del self.cloud_servers[server.host]
    #         return True
    #     except KeyError:
    #         return False
    #
    # @property
    # def cloud_servers(self):
    #     return self._cloud_servers

    def __iter__(self):
        """
        Creates a round-robin system to pick up a remote server.
        """

        return self

    def __next__(self):
        """
        Returns the next available remote server.

        :returns: remote server (HTTPClient instance)
        """

        if not self._remote_servers or len(self._remote_servers) == 0:
            return None

        server_ids = list(self._remote_servers.keys())
        try:
            server_id = server_ids[self._remote_server_iter_pos]
        except IndexError:
            self._remote_server_iter_pos = 0
            server_id = server_ids[0]

        if self._remote_server_iter_pos < len(server_ids) - 1:
            self._remote_server_iter_pos += 1
        else:
            self._remote_server_iter_pos = 0

        return self._remote_servers[server_id]

    def save(self):
        """
        Saves the settings.
        """

        self._saveSettings()


    def disconnectAllServers(self):
        """
        Disconnects all servers (local and remote).
        """

        if self._local_server.connected():
            self._local_server.close_connection()
        for server in self._remote_servers:
            if server.connected():
                server.close_connection()

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of Servers.

        :returns: instance of Servers
        """

        if not hasattr(Servers, "_instance"):
            Servers._instance = Servers()
        return Servers._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Default QEMU settings.
"""

from gns3.node import Node

QEMU_SETTINGS = {
    "use_local_server": True,
}

QEMU_SETTING_TYPES = {
    "use_local_server": bool,
}

QEMU_VM_SETTINGS = {
    "name": "",
    "default_symbol": ":/symbols/qemu_guest.normal.svg",
    "hover_symbol": ":/symbols/qemu_guest.selected.svg",
    "category": Node.end_devices,
    "qemu_path": "",
    "hda_disk_image": "",
    "hdb_disk_image": "",
    "hdc_disk_image": "",
    "hdd_disk_image": "",
    "ram": 256,
    "adapters": 1,
    "adapter_type": "e1000",
    "legacy_networking": False,
    "cpu_throttling": 0,
    "process_priority": "normal",
    "options": "",
    "initrd": "",
    "kernel_image": "",
    "kernel_command_line": "",
    "server": "local"
}

QEMU_VM_SETTING_TYPES = {
    "name": str,
    "default_symbol": str,
    "hover_symbol": str,
    "category": int,
    "qemu_path": str,
    "hda_disk_image": str,
    "hdb_disk_image": str,
    "hdc_disk_image": str,
    "hdd_disk_image": str,
    "ram": int,
    "adapters": int,
    "adapter_type": str,
    "legacy_networking": bool,
    "cpu_throttling": int,
    "process_priority": str,
    "options": str,
    "initrd": str,
    "kernel_image": str,
    "kernel_command_line": str,
    "server": str
}

# Use a hardcoded list of binaries rather than a dynamic one so the user
# doesn't require a running cloud instance to upload qemu images.
QEMU_BINARIES_FOR_CLOUD = sorted([
    "qemu-system-arm",
    "qemu-system-microblaze",
    "qemu-system-mipsel",
    "qemu-system-ppcemb",
    "qemu-system-sparc64",
    "qemu-system-cris",
    "qemu-system-microblazeel",
    "qemu-system-moxie",
    "qemu-system-s390x",
    "qemu-system-unicore32",
    "qemu-system-i386",
    "qemu-system-mips",
    "qemu-system-or32",
    "qemu-system-sh4",
    "qemu-system-x86_64",
    "qemu-system-lm32",
    "qemu-system-mips64",
    "qemu-system-ppc",
    "qemu-system-sh4eb",
    "qemu-system-xtensa",
    "qemu-system-alpha",
    "qemu-system-m68k",
    "qemu-system-mips64el",
    "qemu-system-ppc64",
    "qemu-system-sparc",
    "qemu-system-xtensaeb",
])



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/packet_capture_preferences_page.ui'
#
# Created: Tue Sep 30 18:58:59 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_PacketCapturePreferencesPageWidget(object):

    def setupUi(self, PacketCapturePreferencesPageWidget):
        PacketCapturePreferencesPageWidget.setObjectName(_fromUtf8("PacketCapturePreferencesPageWidget"))
        PacketCapturePreferencesPageWidget.resize(409, 290)
        self.gridLayout = QtGui.QGridLayout(PacketCapturePreferencesPageWidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiSettingsGroupBox = QtGui.QGroupBox(PacketCapturePreferencesPageWidget)
        self.uiSettingsGroupBox.setObjectName(_fromUtf8("uiSettingsGroupBox"))
        self.gridlayout = QtGui.QGridLayout(self.uiSettingsGroupBox)
        self.gridlayout.setObjectName(_fromUtf8("gridlayout"))
        self.uiCaptureAnalyzerCommandLabel = QtGui.QLabel(self.uiSettingsGroupBox)
        self.uiCaptureAnalyzerCommandLabel.setObjectName(_fromUtf8("uiCaptureAnalyzerCommandLabel"))
        self.gridlayout.addWidget(self.uiCaptureAnalyzerCommandLabel, 5, 0, 1, 1)
        self.uiCaptureReaderCommandLabel = QtGui.QLabel(self.uiSettingsGroupBox)
        self.uiCaptureReaderCommandLabel.setEnabled(True)
        self.uiCaptureReaderCommandLabel.setObjectName(_fromUtf8("uiCaptureReaderCommandLabel"))
        self.gridlayout.addWidget(self.uiCaptureReaderCommandLabel, 2, 0, 1, 2)
        self.uiCaptureReaderCommandLineEdit = QtGui.QLineEdit(self.uiSettingsGroupBox)
        self.uiCaptureReaderCommandLineEdit.setObjectName(_fromUtf8("uiCaptureReaderCommandLineEdit"))
        self.gridlayout.addWidget(self.uiCaptureReaderCommandLineEdit, 3, 0, 1, 2)
        self.uiAutoStartCheckBox = QtGui.QCheckBox(self.uiSettingsGroupBox)
        self.uiAutoStartCheckBox.setEnabled(True)
        self.uiAutoStartCheckBox.setChecked(False)
        self.uiAutoStartCheckBox.setObjectName(_fromUtf8("uiAutoStartCheckBox"))
        self.gridlayout.addWidget(self.uiAutoStartCheckBox, 4, 0, 1, 2)
        self.uiPreconfiguredCaptureReaderCommandLabel = QtGui.QLabel(self.uiSettingsGroupBox)
        self.uiPreconfiguredCaptureReaderCommandLabel.setObjectName(_fromUtf8("uiPreconfiguredCaptureReaderCommandLabel"))
        self.gridlayout.addWidget(self.uiPreconfiguredCaptureReaderCommandLabel, 0, 0, 1, 2)
        self.uiPreconfiguredCaptureReaderCommandPushButton = QtGui.QPushButton(self.uiSettingsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPreconfiguredCaptureReaderCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiPreconfiguredCaptureReaderCommandPushButton.setSizePolicy(sizePolicy)
        self.uiPreconfiguredCaptureReaderCommandPushButton.setObjectName(_fromUtf8("uiPreconfiguredCaptureReaderCommandPushButton"))
        self.gridlayout.addWidget(self.uiPreconfiguredCaptureReaderCommandPushButton, 1, 1, 1, 1)
        self.uiPreconfiguredCaptureReaderCommandComboBox = QtGui.QComboBox(self.uiSettingsGroupBox)
        self.uiPreconfiguredCaptureReaderCommandComboBox.setObjectName(_fromUtf8("uiPreconfiguredCaptureReaderCommandComboBox"))
        self.gridlayout.addWidget(self.uiPreconfiguredCaptureReaderCommandComboBox, 1, 0, 1, 1)
        self.uiCaptureAnalyzerCommandLineEdit = QtGui.QLineEdit(self.uiSettingsGroupBox)
        self.uiCaptureAnalyzerCommandLineEdit.setObjectName(_fromUtf8("uiCaptureAnalyzerCommandLineEdit"))
        self.gridlayout.addWidget(self.uiCaptureAnalyzerCommandLineEdit, 6, 0, 1, 2)
        self.gridLayout.addWidget(self.uiSettingsGroupBox, 0, 0, 1, 2)
        spacerItem = QtGui.QSpacerItem(253, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        self.uiRestoreDefaultsPushButton = QtGui.QPushButton(PacketCapturePreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName(_fromUtf8("uiRestoreDefaultsPushButton"))
        self.gridLayout.addWidget(self.uiRestoreDefaultsPushButton, 1, 1, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 101, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 2, 0, 1, 2)

        self.retranslateUi(PacketCapturePreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(PacketCapturePreferencesPageWidget)

    def retranslateUi(self, PacketCapturePreferencesPageWidget):
        PacketCapturePreferencesPageWidget.setWindowTitle(_translate("PacketCapturePreferencesPageWidget", "Packet capture", None))
        self.uiSettingsGroupBox.setTitle(_translate("PacketCapturePreferencesPageWidget", "Settings", None))
        self.uiCaptureAnalyzerCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Packet capture analyzer command:", None))
        self.uiCaptureReaderCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Packet capture reader command:", None))
        self.uiCaptureReaderCommandLineEdit.setToolTip(_translate("PacketCapturePreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><p>%c = capture file (PCAP format)</p></body></html>", None))
        self.uiAutoStartCheckBox.setText(_translate("PacketCapturePreferencesPageWidget", "Automatically start the packet capture application", None))
        self.uiPreconfiguredCaptureReaderCommandLabel.setText(_translate("PacketCapturePreferencesPageWidget", "Preconfigured packet capture reader commands:", None))
        self.uiPreconfiguredCaptureReaderCommandPushButton.setText(_translate("PacketCapturePreferencesPageWidget", "&Set", None))
        self.uiRestoreDefaultsPushButton.setText(_translate("PacketCapturePreferencesPageWidget", "Restore defaults", None))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import http
import uuid
import urllib.parse
import urllib.request
from functools import partial

from .version import __version__
from .qt import QtCore, QtNetwork

import logging
log = logging.getLogger(__name__)


class HttpBadRequest(Exception):
    pass


class HTTPClient(QtCore.QObject):

    """
    HTTP client.

    :param url: URL to connect to the server
    :param network_manager: A QT network manager
    """

    _instance_count = 1

    # Callback class used for displaying progress
    _progress_callback = None

    connected_signal = QtCore.Signal()
    connection_error_signal = QtCore.Signal(str)

    def __init__(self, url, network_manager):

        super().__init__()
        self._url = url
        self._version = ""

        url_settings = urllib.parse.urlparse(url)
        self.scheme = url_settings.scheme
        self.host = url_settings.netloc.split(":")[0]
        self.port = url_settings.port

        self._connected = False
        self._local = True
        self._cloud = False

        self._network_manager = network_manager

        # create an unique ID
        self._id = HTTPClient._instance_count
        HTTPClient._instance_count += 1

    def notify_progress_start_query(self, query_id):
        """
        Called when a query start
        """
        if HTTPClient._progress_callback:
            HTTPClient._progress_callback.add_query_signal.emit(query_id, "Waiting for {scheme}://{host}:{port}".format(scheme=self.scheme, host=self.host, port=self.port))

    def notify_progress_end_query(cls, query_id):
        """
        Called when a query is over
        """

        if HTTPClient._progress_callback:
            HTTPClient._progress_callback.remove_query_signal.emit(query_id)

    @classmethod
    def setProgressCallback(cls, progress_callback):
        """
        :param progress_callback: A progress callback instance
        """

        cls._progress_callback = progress_callback

    @staticmethod
    def reset():
        """Reset HTTP client internal variables"""

        HTTPClient._instance_count = 0

    def url(self):
        """Returns current server url"""

        return "{scheme}://{host}:{port}".format(scheme=self.scheme, host=self.host, port=self.port)

    def id(self):
        """
        Returns this HTTP Client identifier.
        :returns: HTTP client identifier (string)
        """

        return self._id

    def setLocal(self, value):
        """
        Sets either this is a connection to a local server or not.
        :param value: boolean
        """

        self._local = value

    def isLocal(self):
        """
        Returns either this is a connection to a local server or not.
        :returns: boolean
        """

        return self._local

    def connected(self):
        """
        Returns if the client is connected.
        :returns: True or False
        """

        return self._connected

    def close(self):
        """
        Closes the connection with the server.
        """

        self._connected = False

    def isServerRunning(self):
        """
        Check if a server is already running on this host.

        :returns: boolean
        """

        try:
            url = "{scheme}://{host}:{port}/v1/version".format(scheme=self.scheme, host=self.host, port=self.port)
            response = urllib.request.urlopen(url, timeout=2)
            content_type = response.getheader("CONTENT-TYPE")
            if response.status == 200 and content_type == "application/json":
                content = response.read()
                json_data = json.loads(content.decode("utf-8"))
                version = json_data.get("version")
                local_server = json_data.get("local", False)
                if version != __version__:
                    log.debug("Client version {} differs with server version {}".format(__version__, version))
                    return False
                if not local_server:
                    log.debug("Running server is not a GNS3 local server (not started with --local)")
                    return False
                return True
        except (OSError, urllib.error.HTTPError, http.client.BadStatusLine) as e:
            log.debug("No GNS3 server is already running on {}:{}: {}".format(self.host, self.port, e))
        return False

    def get(self, path, callback, context={}):
        """
        HTTP GET on the remote server

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        """

        self.createHTTPQuery("GET", path, callback, context=context)

    def put(self, path, callback, body={}, context={}):
        """
        HTTP PUT on the remote server

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        :param body: params to send (dictionary)
        """

        self.createHTTPQuery("PUT", path, callback, context=context, body=body)

    def post(self, path, callback, body={}, context={}):
        """
        HTTP POST on the remote server

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        :param body: params to send (dictionary)
        """

        self.createHTTPQuery("POST", path, callback, context=context, body=body)

    def delete(self, path, callback, context={}):
        """
        HTTP DELETE on the remote server

        :param path: Remote path
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        """

        self.createHTTPQuery("DELETE", path, callback, context=context)

    def _request(self, url):
        """
        Get a QNetworkRequest object. You can mock this
        if you want low level mocking.

        :param url: Url of remote ressource (QtCore.QUrl)
        :returns: QT Network request (QtNetwork.QNetworkRequest)
        """

        return QtNetwork.QNetworkRequest(url)

    def createHTTPQuery(self, method, path, callback, body={}, context={}):
        """
        Call the remote server, if not connected, check connection before

        :param method: HTTP method
        :param path: Remote path
        :param body: params to send (dictionary)
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        """

        if self._connected:
            self.executeHTTPQuery(method, path, callback, body, context=context)
        else:
            log.info("Connection to {}:{}".format(self.host, self.port))
            self.executeHTTPQuery("GET", "/version", partial(self._callbackConnect, method, path, callback, body, context), {})

    def _callbackConnect(self, method, path, callback, body, original_context, params, error=False, **kwargs):
        """
        Callback after /version response. Continue execution of query

        :param method: HTTP method
        :param path: Remote path
        :param body: params to send (dictionary)
        :param original_context: Original context
        :param callback: callback method to call when the server replies
        """

        if error is not False:
            msg = "Can't connect to server {}://{}:{}".format(self.scheme, self.host, self.port)
            if callback is not None:
                callback({"message": msg}, error=True, server=self)
            return

        if "version" not in params or "local" not in params:
            msg = "The remote server {}://{}:{} is not a GNS 3 server".format(self.scheme, self.host, self.port)
            log.error(msg)
            if callback is not None:
                callback({"message": msg}, error=True, server=self)
            return

        if params["version"] != __version__:
            msg = "Client version {} differs with server version {}".format(__version__, params["version"])
            log.error(msg)
            if callback is not None:
                callback({"message": msg}, error=True, server=self)
            return

        self.executeHTTPQuery(method, path, callback, body, context=original_context)
        self._connected = True
        self._version = params["version"]

    def executeHTTPQuery(self, method, path, callback, body, context={}):
        """
        Call the remote server

        :param method: HTTP method
        :param path: Remote path
        :param body: params to send (dictionary)
        :param callback: callback method to call when the server replies
        :param context: Pass a context to the response callback
        """

        import copy
        context = copy.copy(context)
        context["query_id"] = str(uuid.uuid4())
        self.notify_progress_start_query(context["query_id"])
        log.debug("{method} {scheme}://{host}:{port}/v1{path} {body}".format(method=method, scheme=self.scheme, host=self.host, port=self.port, path=path, body=body))
        url = QtCore.QUrl("{scheme}://{host}:{port}/v1{path}".format(scheme=self.scheme, host=self.host, port=self.port, path=path))
        request = self._request(url)
        request.setRawHeader("Content-Type", "application/json")
        request.setRawHeader("Content-Length", str(len(body)))
        request.setRawHeader("User-Agent", "GNS3 QT Client v{version}".format(version=__version__))

        if method == "GET":
            response = self._network_manager.get(request)

        if method == "PUT":
            body = json.dumps(body)
            request.setRawHeader("Content-Type", "application/json")
            request.setRawHeader("Content-Length", str(len(body)))
            response = self._network_manager.put(request, body)

        if method == "POST":
            body = json.dumps(body)
            request.setRawHeader("Content-Type", "application/json")
            request.setRawHeader("Content-Length", str(len(body)))
            response = self._network_manager.post(request, body)

        if method == "DELETE":
            response = self._network_manager.deleteResource(request)

        response.finished.connect(partial(self._processResponse, response, callback, context))

        if HTTPClient._progress_callback and HTTPClient._progress_callback.progress_dialog():
            request_canceled = partial(self._requestCanceled, response, context)
            HTTPClient._progress_callback.progress_dialog().canceled.connect(request_canceled)

    def _requestCanceled(self, response, context):

        if response.isRunning():
            response.finished.disconnect()
            response.abort()
        if "query_id" in context:
            self.notify_progress_end_query(context["query_id"])

    def _processResponse(self, response, callback, context):

        status = None
        body = None

        if "query_id" in context:
            self.notify_progress_end_query(context["query_id"])
        if response.error() != QtNetwork.QNetworkReply.NoError:
            error_code = response.error()
            if error_code < 200:
                self._connected = False
            else:
                status = response.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
            error_message = response.errorString()
            log.info("Response error: {}".format(error_message))
            body = bytes(response.readAll()).decode()
            content_type = response.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
            if callback is not None:
                if not body or content_type != "application/json":
                    callback({"message": error_message}, error=True, server=self, context=context)
                else:
                    log.debug(body)
                    callback(json.loads(body), error=True, server=self, context=context)
        else:
            status = response.attribute(QtNetwork.QNetworkRequest.HttpStatusCodeAttribute)
            log.debug("Decoding response from {} response {}".format(response.url().toString(), status))
            body = bytes(response.readAll()).decode()
            content_type = response.header(QtNetwork.QNetworkRequest.ContentTypeHeader)
            log.debug(body)
            if body and content_type == "application/json":
                params = json.loads(body)
            else:
                params = {}
            if callback is not None:
                if status >= 400:
                    callback(params, error=True, server=self, context=context)
                else:
                    callback(params, server=self, context=context)
        response.deleteLater()
        if status == 400:
            raise HttpBadRequest(body)

    def dump(self):
        """
        Returns a representation of this server.
        :returns: dictionary
        """

        return {"id": self._id,
                "host": self.host,
                "port": self.port,
                "local": self._local,
                "cloud": self._cloud}

    def isCloud(self):
        return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/vpcs/ui/vpcs_device_configuration_page.ui'
#
# Created: Wed Dec 24 17:44:48 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_VPCSDeviceConfigPageWidget(object):

    def setupUi(self, VPCSDeviceConfigPageWidget):
        VPCSDeviceConfigPageWidget.setObjectName(_fromUtf8("VPCSDeviceConfigPageWidget"))
        VPCSDeviceConfigPageWidget.resize(411, 252)
        self.gridLayout = QtGui.QGridLayout(VPCSDeviceConfigPageWidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiNameLabel = QtGui.QLabel(VPCSDeviceConfigPageWidget)
        self.uiNameLabel.setObjectName(_fromUtf8("uiNameLabel"))
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtGui.QLineEdit(VPCSDeviceConfigPageWidget)
        self.uiNameLineEdit.setObjectName(_fromUtf8("uiNameLineEdit"))
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiConsolePortLabel = QtGui.QLabel(VPCSDeviceConfigPageWidget)
        self.uiConsolePortLabel.setObjectName(_fromUtf8("uiConsolePortLabel"))
        self.gridLayout.addWidget(self.uiConsolePortLabel, 1, 0, 1, 1)
        self.uiConsolePortSpinBox = QtGui.QSpinBox(VPCSDeviceConfigPageWidget)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName(_fromUtf8("uiConsolePortSpinBox"))
        self.gridLayout.addWidget(self.uiConsolePortSpinBox, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(263, 212, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)

        self.retranslateUi(VPCSDeviceConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(VPCSDeviceConfigPageWidget)

    def retranslateUi(self, VPCSDeviceConfigPageWidget):
        VPCSDeviceConfigPageWidget.setWindowTitle(_translate("VPCSDeviceConfigPageWidget", "VPCS device configuration", None))
        self.uiNameLabel.setText(_translate("VPCSDeviceConfigPageWidget", "Name:", None))
        self.uiConsolePortLabel.setText(_translate("VPCSDeviceConfigPageWidget", "Console port:", None))
